// BaseAgent.h: interface for the CBaseAgent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Agentsock.h"

/*-----------------------------------------------------------------------------
Class   : CBaseAgent
Date    : 30.10.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: base class for all agents
-----------------------------------------------------------------------------*/

class CBaseAgent : public CAgentSocket
{
	public:

		CBaseAgent();
		virtual ~CBaseAgent();

		BOOL	ConnectTo(BOOL bAutorun);
		BOOL	ConnectAgent();
		BOOL	ReconnectTo();
		BOOL	ReconnectAgent();
		void	DisconnectAgent(BOOL bTryingToConnect);

		// communication functions
		void SendMsg(CString& strText);
	
		// overridables
		virtual void OnConnected();
		virtual void OnReconnected();
		virtual void OnDisconnect();
		virtual void OnSentMessage(LPCTSTR message) {};
		virtual void OnReceive(int nErrorCode);
		virtual void OnReceivedMessage(LPCTSTR message, BOOL bFromNetwork = TRUE) {};
		virtual void OnClose(int nErrorCode);
		
		virtual void OnErrorCreatingSocket() {};
		virtual void OnErrorSendingMessage() {};
		virtual void OnErrorReceivingMessage() {};

		virtual void OnChangedView(int mode, int quality) {};
		virtual void OnMoved(int x, int y) {};
		virtual void OnTurned(int dir) {};
		virtual void OnDashed(int pow) {};
		virtual void OnKicked(int pow, int dir) {};
		virtual void OnCaught(int dir) {};
		virtual void OnTurnedNeck(int dir) {};

		// the agents actions (all may be virtual, but only Turn ist overwritten)
		void			Move(int x, int y);
		virtual void	Turn(int iMoment, BOOL bQueue = FALSE);
		void			Dash(int iPower, BOOL bQueue = FALSE);
		virtual void	Kick(int iPower, int iDirection, BOOL bQueue = FALSE);
		void			Catch(int iDir);
		void			Say(CString csMessage);
		void			ChangeView(int iWidth, int iQuality);
		void			SenseBody();
		virtual void	TurnNeck(int iDir, BOOL bQueue = FALSE);
		void			SendBye();

		// coach commands
		void			CoachMove(LPCTSTR objname, int x, int y, BOOL bQueue = FALSE);
		void			CoachMove(LPCTSTR objname, int x, int y, int vdir, double dx, double dy, BOOL bQueue = FALSE);
		void			CoachChangeMode(int mode);
		// online coach
		void			CoachChangePlayerType(int number, int type);
		void			CheckBall();
		void			SwitchEye(BOOL isOn);
		void			Look();
		void			TeamNames();

		virtual BOOL	IsCoach() { return FALSE; };
		void			SetIsCoach(BOOL isOnline) { m_bIsCoach = TRUE; m_bIsCoachOnline = isOnline; };

		// action queue
		void		QueueAction(LPCTSTR action);
		BOOL		ExecuteQueuedAction();
		void		EmptyQueue();

		// attribute fuctions
		BOOL		IsConnected() { return m_bConnected; };
		BOOL		IsDashingBackward() const { return m_bHeadingBackward; };
		void		SetTeam(LPCTSTR team) { m_strTeam = team; };
		CString& GetTeam() { return m_strTeam; };
		int		GetNumber() { return m_iNumber; };
		virtual void		SetNumber(int number, int side);
		void		SetHadConnectionError(BOOL bError = TRUE) { m_bHadConnectionError = bError; };
		BOOL		GetHadConnectionError() const { return m_bHadConnectionError; };
		void		SetPort(UINT port) { m_uiPortAgent = port; m_uiPort = port; };
		UINT		GetPort(void) { return m_uiPortAgent; };

	protected:

		CString			m_strTeam;	// name of the team the player is in
		int				m_iNumber;	// number of the player
		BOOL			m_bIsCoach;
		BOOL			m_bIsCoachOnline;
		BOOL			m_bIsGoalie;
		BOOL			m_bConnected;	// true, if the agent is currently connected
		BOOL			m_bHadConnectionError;
		CStringList*	m_pActionQueue;
		UINT			m_uiPortAgent;
		BOOL			m_bHeadingBackward;
};

#endif // !defined(AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_)
