// BasisDoc.cpp : implementation of the CBasisDoc class
//
#include "stdafx.h"
#include "stdbnx.h"
#include "Basis.h"

#include "BaseAgent.h"
#include "TDoingAgent.h"
#include "TCoachAgent.h"
#include "BasisDoc.h"
#include "BasisView.h"
#include "TPerceptionView.h"
#include "TMonitorSock.h"

#include <stdlib.h>
#include <time.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc

IMPLEMENT_DYNCREATE(CBasisDoc, CDocument)

BEGIN_MESSAGE_MAP(CBasisDoc, CDocument)
	//{{AFX_MSG_MAP(CBasisDoc)
	ON_COMMAND(ID_CONNECT, OnConnect)
	ON_COMMAND(ID_DISCONNECT, OnDisconnect)
	ON_UPDATE_COMMAND_UI(ID_DISCONNECT, OnUpdateDisconnect)
	ON_UPDATE_COMMAND_UI(ID_CONNECT, OnUpdateConnect)
	ON_COMMAND(ID_VIEW_NOGRAPHICS, OnViewNographics)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NOGRAPHICS, OnUpdateViewNographics)
	ON_COMMAND(ID_TEAM_LEFT, OnTeamLeft)
	ON_UPDATE_COMMAND_UI(ID_TEAM_LEFT, OnUpdateTeamLeft)
	ON_COMMAND(ID_TEAM_RIGHT, OnTeamRight)
	ON_UPDATE_COMMAND_UI(ID_TEAM_RIGHT, OnUpdateTeamRight)
	ON_COMMAND(ID_KNOWLEDGE_DORER, OnKnowledgeDorer)
	ON_UPDATE_COMMAND_UI(ID_KNOWLEDGE_DORER, OnUpdateKnowledgeDorer)
	ON_COMMAND(ID_KNOWLEDGE_MAES, OnKnowledgeMaes)
	ON_UPDATE_COMMAND_UI(ID_KNOWLEDGE_MAES, OnUpdateKnowledgeMaes)
	ON_COMMAND(ID_GOAL_TRACKING_ON, OnGoalTrackingOn)
	ON_UPDATE_COMMAND_UI(ID_GOAL_TRACKING_ON, OnUpdateGoalTrackingOn)
	ON_COMMAND(ID_USE_FUZZY_VALUES, OnUseFuzzyValues)
	ON_UPDATE_COMMAND_UI(ID_USE_FUZZY_VALUES, OnUpdateUseFuzzyValues)
	ON_COMMAND(ID_ADAPTION_ON, OnAdaptionOn)
	ON_UPDATE_COMMAND_UI(ID_ADAPTION_ON, OnUpdateAdaptionOn)
	ON_COMMAND(ID_KNOWLEDGE_DORER2, OnKnowledgeDorer2)
	ON_UPDATE_COMMAND_UI(ID_KNOWLEDGE_DORER2, OnUpdateKnowledgeDorer2)
	ON_COMMAND(IDC_START_AGENT, OnStartAgent)
	ON_UPDATE_COMMAND_UI(IDC_START_AGENT, OnUpdateStartAgent)
	ON_COMMAND(IDC_STOP_AGENT, OnStopAgent)
	ON_UPDATE_COMMAND_UI(IDC_STOP_AGENT, OnUpdateStopAgent)
	ON_COMMAND(ID_CONNECT_TO, OnConnectTo)
	ON_UPDATE_COMMAND_UI(ID_CONNECT_TO, OnUpdateConnectTo)
	ON_COMMAND(ID_RECONNECT, OnReconnect)
	ON_UPDATE_COMMAND_UI(ID_RECONNECT, OnUpdateReconnect)
	ON_COMMAND(ID_RECONNECT_TO, OnReconnectTo)
	ON_UPDATE_COMMAND_UI(ID_RECONNECT_TO, OnUpdateReconnectTo)
	ON_COMMAND(ID_VIEW_GENETICS, OnViewGenetics)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GENETICS, OnUpdateViewGenetics)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc construction/destruction

CBasisDoc::CBasisDoc()
{
	m_pBasisView = NULL;
	m_pPerceptionView = NULL;
	m_pKnowledgeView = NULL;
	m_pNetworkView = NULL;
	m_pAgent = NULL;
	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;
	m_bAutorun = FALSE;
	m_iAutorunID = 0;
	m_bSentConnected = FALSE;
	m_pMonitorSock = NULL;
	m_strMonitorAddress = "localhost";
	m_uiMonitorPort	= 6200U;
	
	#ifdef _DEBUG
		m_bNoGraphics = TRUE;
//		m_bNoGraphics = FALSE;
	#else
		m_bNoGraphics = TRUE;
	#endif
}

CBasisDoc::~CBasisDoc()
{
	if (m_pMonitorSock != NULL)
		delete m_pMonitorSock;
}

BOOL CBasisDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// get the pointers of all views
	for(POSITION pos = GetFirstViewPosition(); pos != NULL;)
	{
		CView* pView = GetNextView(pos);
		
		if (pView->IsKindOf(RUNTIME_CLASS(CBasisView)))
			m_pBasisView = (CBasisView*) pView;
		else if (pView->IsKindOf(RUNTIME_CLASS(TPerceptionView)))
			m_pPerceptionView = (TPerceptionView*) pView;
		else if (pView->IsKindOf(RUNTIME_CLASS(TRuleView)))
			m_pKnowledgeView = (TRuleView*) pView;
		else if (pView->IsKindOf(RUNTIME_CLASS(TNetworkView)))
			m_pNetworkView = (TNetworkView*) pView;
	}

	// set all views to display graphics or not
	m_pBasisView->SetNoGraphics(m_bNoGraphics);
	m_pPerceptionView->SetNoGraphics(m_bNoGraphics);
	m_pKnowledgeView->SetNoGraphics(m_bNoGraphics);
	m_pNetworkView->SetNoGraphics(m_bNoGraphics);

	// create a new agent
	m_pAgent = new TDoingAgent(this, m_pBasisView, m_pPerceptionView
									, m_pNetworkView, m_pKnowledgeView);

	SetKnowledgeType(DORER2_KNOWLEDGE);
	// try to open the default rule file
	if (!m_pKnowledgeView->OpenStandardFile("launch\\ruleDefender.txt"))
		m_pKnowledgeView->OpenStandardFile("ruleDefender.txt");

	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;

	return TRUE;
}

void CBasisDoc::SetIsCoach(BOOL isOnline)
{
	// create a coach agent
	delete m_pAgent;

	m_pAgent = new TCoachAgent(this, m_pBasisView, m_pPerceptionView
									, m_pNetworkView, m_pKnowledgeView);

	SetKnowledgeType(DORER2_KNOWLEDGE);
	// try to open the default rule file
	if (!m_pKnowledgeView->OpenStandardFile("launch\\ruleCoach.txt"))
		m_pKnowledgeView->OpenStandardFile("ruleCoach.txt");
	
	m_pAgent->SetIsCoach(isOnline);
}

void CBasisDoc::DeleteContents() 
{
	if (m_pAgent != NULL)
	{
		// delete the agent
		Disconnect();
		delete m_pAgent;
		m_pAgent = NULL;
	}

	CDocument::DeleteContents();
	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;
}



/////////////////////////////////////////////////////////////////////////////
// CBasisDoc serialization

void CBasisDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc diagnostics

#ifdef _DEBUG
void CBasisDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CBasisDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CBasisDoc commands

void CBasisDoc::SetAutorun(BOOL bAutorun, int iAutorunID, int iPlayerID)
{
	// called, when parameter /auto is passed
	m_bAutorun = bAutorun;
	m_iAutorunID = iAutorunID;
	if (m_bAutorun && (m_pMonitorSock == NULL))
	{
		// create socket and connect to soccermonitor
		m_pMonitorSock = new TMonitorSock(this);
		m_pMonitorSock->Create();
		m_pMonitorSock->Connect(m_strMonitorAddress, m_uiMonitorPort);
		// may have  WSAECONNREFUSED
	}

	// set the number of the player, so that it eventually logs on as goalie
	// side parameter is not important
	if (iPlayerID == 0)
		// it is a game for the dis
		m_pAgent->SetOrdinaryGame(TRUE);
	else
		// it is a game for robocup
		m_pAgent->SetNumber(iPlayerID, 0);
}

void CBasisDoc::AutorunAgentReady()
{
	int				len;
	CString			result;
	const char*		buffer;

	// called when the first see perception arrived
	if (m_bAutorun && !m_bSentConnected)
	{
		// send the connected tag
		m_pMonitorSock->SendConnected();
		m_bSentConnected = TRUE;
	
		// send the id of the agent
		m_pMonitorSock->Send(&m_iAutorunID, sizeof(m_iAutorunID));

		// send the names of the result columns
		result = m_pAgent->GetResultsInitial();
		len = result.GetLength() + 1;
		m_pMonitorSock->Send(&len, sizeof(len));
		buffer = (LPCTSTR) result;
		m_pMonitorSock->Send(buffer, len);
	}
}

void CBasisDoc::AutorunReceivedTerminate()
{
	m_pBasisView->StopAgentTimer();
	m_pAgent->ReceivedTerminate();
}

void CBasisDoc::AutorunSendResults()
{
	int				len;
	CString			result;
	const char*		buffer;

	// called, when the soccermonitor sent the MSG_RESULTS message
	// send message header
	m_pMonitorSock->SendResults();

	// send the results
	result = m_pAgent->GetResults();
	len = result.GetLength() + 1;
	m_pMonitorSock->Send(&len, sizeof(len));
	buffer = (LPCTSTR) result;
	m_pMonitorSock->Send(buffer, len);
}

void CBasisDoc::Terminate()
{	
	// exit the program (gruel, isn't it)
	SetModifiedFlag(FALSE);
	theApp.m_pMainWnd->SendMessage(WM_CLOSE);
}

TNetworkParams* CBasisDoc::GetNetworkParams()
{
	return m_pAgent->GetNetworkParams();
}

void CBasisDoc::SetTeamname(LPCTSTR name)
{
	m_pAgent->SetTeam(name);
}

void CBasisDoc::SetNumber(int number)
{
	m_pAgent->SetNumber(number, 0);
}

void CBasisDoc::SetRuleFile(LPCTSTR filename)
{
	// load the specified default file
	if (!m_pKnowledgeView->OpenStandardFile(filename))
		m_pKnowledgeView->OpenStandardFile(CString("save\\") + filename); 
}

void CBasisDoc::SetMonitorAddress(LPCTSTR address, UINT iPort)
{
	m_strMonitorAddress = address;
	m_uiMonitorPort = iPort;
}

void CBasisDoc::SetServerAddress(LPCTSTR address)
{
	m_pAgent->SetServerAddress(address);
}

void CBasisDoc::SetServerPort(int iPort)
{
	m_pAgent->SetPort(iPort);
}

void CBasisDoc::SetGoalTrackingOn(BOOL bVal)
{
	m_pAgent->SetGoalTrackingOn(bVal);
}

void CBasisDoc::SetUseFuzzyValues(BOOL bVal)
{
	m_pAgent->SetUseFuzzyValues(bVal);
}

void CBasisDoc::SetFuzzyRound(double dVal)
{
	m_pAgent->SetFuzzyRound(dVal);
}

void CBasisDoc::SetAdaptActivationAreaOn(BOOL bVal)
{
	m_pAgent->SetAdaptActivationAreaOn(bVal);
}

void CBasisDoc::InsertStringToRule(LPCTSTR string)
{
	m_pKnowledgeView->InsertString(string);
}

void CBasisDoc::SetCombo(CComboBox* pCombo, BOOL bWhich)
{
	m_pAgent->SetCombo(pCombo, bWhich);
}

void CBasisDoc::SetNoise(double noise)
{
	m_pAgent->SetNoise(noise);
}

void CBasisDoc::OnConnectTo() 
{
	// connect as player
	if (m_pAgent->ConnectTo(m_bAutorun))
	{
		m_bTryingToConnect = TRUE;
		
		// no need for start connection timer when autorunning ?
		if (!m_bAutorun)
			m_pBasisView->StartConnectionTimer();
	
		// start the agent
		m_pBasisView->StartAgentTimer();
	}
}

void CBasisDoc::OnConnect() 
{
	BOOL	bSuccess;

	// connect as player, but ask for server, if an error has previously occured
	if (m_pAgent->GetHadConnectionError())
		bSuccess = m_pAgent->ConnectTo(m_bAutorun);
	else
		bSuccess = m_pAgent->ConnectAgent();
	
	if (bSuccess)
	{
		m_bTryingToConnect = TRUE;
		
		// no need for start connection timer when autorunning ?
		if (!m_bAutorun)
			m_pBasisView->StartConnectionTimer();
	
		// start the agent
		m_pBasisView->StartAgentTimer();
	}
}

// function that can be called from outside
void CBasisDoc::Connect() 
{
	// if this message is a little bit late processed
	if (m_bConnected)
		return;
	OnConnect();
}

// function that can be called from outside
void CBasisDoc::ReConnect() 
{
	// if this message is a little bit late processed
	if (m_bConnected)
		return;
	OnReconnect();
}

void CBasisDoc::OnReconnectTo() 
{
	// reconnect as
	if (m_pAgent->ReconnectTo())
	{
		m_bTryingToConnect = TRUE;
		
		// no need for start connection timer when autorunning ?
		if (!m_bAutorun)
			m_pBasisView->StartConnectionTimer();
	
		// start the agent
		m_pBasisView->StartAgentTimer();
	}
}

void CBasisDoc::OnReconnect() 
{
	// reconnect
	if (m_pAgent->ReconnectAgent())
	{
		m_bTryingToConnect = TRUE;
		
		// no need for start connection timer when autorunning ?
		if (!m_bAutorun)
			m_pBasisView->StartConnectionTimer();
	
		// start the agent
		m_pBasisView->StartAgentTimer();
	}
}

void CBasisDoc::SetConnected()
{
	if (!m_bConnected)
	{
		m_pBasisView->StopConnectionTimer();
		m_bConnected = TRUE;
		m_bTryingToConnect = FALSE;
	}
}

void CBasisDoc::OnConnectionTimeout()
{
	// we got a timeout when trying to connect to the soccerserver
	// called by cbasisview
	m_pAgent->SetHadConnectionError();
	OnDisconnect();
	if (m_bAutorun)
		// leave application
		Terminate();
}

void CBasisDoc::OnConnectionError()
{
	// we got a connection error when trying to connect to the soccerserver
	// called by the agent
	m_pBasisView->StopConnectionTimer();
	OnDisconnect();
	if (m_bAutorun)
		// leave application
		Terminate();
}

void CBasisDoc::OnDisconnect() 
{
	// stop connection
	Disconnect();
	// stop agent
	m_pBasisView->StopAgentTimer();
}

void CBasisDoc::Disconnect() 
{
	// if this message is a little bit late processed
	if (!m_bConnected && !m_bTryingToConnect)
		return;

	// close the socket
	m_pAgent->DisconnectAgent(m_bTryingToConnect);
	m_bTryingToConnect = FALSE;
	m_bConnected = FALSE;
}

void CBasisDoc::OnActionTimer()
{
	m_pAgent->OnTimer(FALSE);
}

void CBasisDoc::OnViewNographics() 
{
	m_bNoGraphics = !m_bNoGraphics;
	m_pBasisView->SetNoGraphics(m_bNoGraphics);
	m_pPerceptionView->SetNoGraphics(m_bNoGraphics);
	m_pKnowledgeView->SetNoGraphics(m_bNoGraphics);
	m_pNetworkView->SetNoGraphics(m_bNoGraphics);
}


void CBasisDoc::OnTeamRight() 
{
	// the player should be in team right
	m_pAgent->SetTeam(OTHER_TEAM);
	m_pAgent->SetSide(RIGHT_TO_LEFT);
}

void CBasisDoc::OnTeamLeft() 
{
	// the player should be in team left
	m_pAgent->SetTeam(MY_TEAM);
	m_pAgent->SetSide(LEFT_TO_RIGHT);
}

void CBasisDoc::SetKnowledgeType(int type)
{
	m_pAgent->SetKnowledgeType(type);
}

void CBasisDoc::OnKnowledgeDorer2() 
{
	SetKnowledgeType(DORER2_KNOWLEDGE);
	// reopen the rule file
	m_pKnowledgeView->OpenCurrentFile();
}

void CBasisDoc::OnKnowledgeDorer() 
{
	SetKnowledgeType(DORER_KNOWLEDGE);
	// reopen the rule file
	m_pKnowledgeView->OpenCurrentFile();
}


void CBasisDoc::OnKnowledgeMaes() 
{
	SetKnowledgeType(MAES_KNOWLEDGE);
	// reopen the rule file
	m_pKnowledgeView->OpenCurrentFile();
}

void CBasisDoc::OnGoalTrackingOn() 
{
	m_pAgent->SetGoalTrackingOn(!m_pAgent->GetGoalTrackingOn());
}

void CBasisDoc::OnUseFuzzyValues() 
{
	m_pAgent->SetUseFuzzyValues(!m_pAgent->GetUseFuzzyValues());
}

void CBasisDoc::OnAdaptionOn() 
{
	m_pAgent->SetAdaptActivationAreaOn(!m_pAgent->GetAdaptActivationAreaOn());
}


void CBasisDoc::OnUpdateConnectTo(CCmdUI* pCmdUI) 
{
	if (m_bConnected || m_bTryingToConnect)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CBasisDoc::OnUpdateConnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected || m_bTryingToConnect)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CBasisDoc::OnUpdateReconnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected || m_bTryingToConnect || (m_pAgent->GetNumber() == 0))
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CBasisDoc::OnUpdateReconnectTo(CCmdUI* pCmdUI) 
{
	if (m_bConnected || m_bTryingToConnect)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CBasisDoc::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CBasisDoc::OnUpdateViewNographics(CCmdUI* pCmdUI) 
{
	if (m_bNoGraphics)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisDoc::OnUpdateTeamLeft(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void CBasisDoc::OnUpdateTeamRight(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT)
		pCmdUI->SetRadio(FALSE);
	else
		pCmdUI->SetRadio(TRUE);
}


void CBasisDoc::OnUpdateKnowledgeDorer2(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() == DORER2_KNOWLEDGE)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void CBasisDoc::OnUpdateKnowledgeDorer(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() == DORER_KNOWLEDGE)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void CBasisDoc::OnUpdateKnowledgeMaes(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() == MAES_KNOWLEDGE)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void CBasisDoc::OnUpdateGoalTrackingOn(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() != DORER_KNOWLEDGE)
		pCmdUI->Enable(FALSE);
	else if (m_pAgent->GetGoalTrackingOn())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisDoc::OnUpdateUseFuzzyValues(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() == MAES_KNOWLEDGE)
		pCmdUI->Enable(FALSE);
	else if (m_pAgent->GetUseFuzzyValues())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisDoc::OnUpdateAdaptionOn(CCmdUI* pCmdUI) 
{
	if (m_pAgent->GetKnowledgeType() != DORER_KNOWLEDGE)
		pCmdUI->Enable(FALSE);
	else if (m_pAgent->GetAdaptActivationAreaOn())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

// for blocksworld
BOOL CBasisDoc::GetRunning()
{
	return m_pBasisView->GetRunning();
}

BOOL CBasisDoc::IsBlocksworld()
{
	return m_pAgent->IsBlocksworld();
}

BOOL CBasisDoc::PerformCycle(TNetworkParams* params, int iNoOfCycles)
{
	return m_pAgent->PerformCycle(params, iNoOfCycles);
}

void CBasisDoc::StopAgent()
{
	OnStopAgent();
}

void CBasisDoc::ResetNetwork()
{
	m_pAgent->ResetNetwork();
}

void CBasisDoc::OnStartAgent() 
{
	m_pBasisView->StartAgentTimer();
}

void CBasisDoc::OnStopAgent() 
{
	m_pBasisView->StopAgentTimer();
}

void CBasisDoc::OnUpdateStartAgent(CCmdUI* pCmdUI) 
{
	if (m_pBasisView->GetRunning())
	{
		pCmdUI->Enable(FALSE);
		pCmdUI->SetRadio(TRUE);
	}
	else
	{
		pCmdUI->SetRadio(FALSE);
		pCmdUI->Enable(TRUE);
	}
}

void CBasisDoc::OnUpdateStopAgent(CCmdUI* pCmdUI) 
{
	if (m_pBasisView->GetRunning())
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CBasisDoc::KickSocket()
{
	// the socket sometimes get stuck under NT. A read seems to reawake it
	m_pAgent->OnReceive(0);
}
void CBasisDoc::OnViewGenetics() 
{
	if (m_pAgent->IsCoach())
	{
		((TCoachAgent*) m_pAgent)->ShowGeneticPoolInfo();
	}
}

void CBasisDoc::OnUpdateViewGenetics(CCmdUI* pCmdUI) 
{
	if (m_pAgent->IsCoach())
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}
