// BasisDoc.h : interface of the CBasisDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASISDOC_H__54CD01CF_4540_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_BASISDOC_H__54CD01CF_4540_11D1_B242_0000C09B5CBE__INCLUDED_


#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TDoingAgent;
class CBaseAgent;

class TPerceptionView;
class TRuleView;
class CBasisView;
class TNetworkView;
class TMonitorSock;
class TNetworkParams;

/*-----------------------------------------------------------------------------
Class   : CBasisDoc
Date    : 14.11.97
-------------------------------------------------------------------------------
Base Class: CDocument

Description: _
-----------------------------------------------------------------------------*/

class CBasisDoc : public CDocument
{
protected: // create from serialization only
	CBasisDoc();
	DECLARE_DYNCREATE(CBasisDoc)

// Attributes
public:

private:
	TDoingAgent*	m_pAgent;
	TPerceptionView*	m_pPerceptionView;
	TRuleView*			m_pKnowledgeView;
	CBasisView*			m_pBasisView;
	TNetworkView*		m_pNetworkView;
	BOOL					m_bNoGraphics;

	BOOL				m_bConnected;
	BOOL				m_bTryingToConnect;
	BOOL				m_bAutorun;
	int				m_iAutorunID;
	BOOL				m_bSentConnected;
	CString			m_strMonitorAddress;
	UINT				m_uiMonitorPort;
	TMonitorSock*	m_pMonitorSock;

// Operations
public:

	TDoingAgent*	GetAgent(void) { return m_pAgent; };
	TNetworkParams* GetNetworkParams();
	void				Connect();
	void				ReConnect();
	void				Disconnect();
	void				SetRuleFile(LPCTSTR filename);
	void				SetTeamname(LPCTSTR team);
	void				SetNumber(int number);
	void				SetMonitorAddress(LPCTSTR address, UINT port);
	void				SetServerAddress(LPCTSTR address);
	void				SetServerPort(int iPort);
	void				SetKnowledgeType(int type);
	void				SetGoalTrackingOn(BOOL bVal);
	void				SetAdaptActivationAreaOn(BOOL bVal);
	void				SetUseFuzzyValues(BOOL bVal);
	void				SetFuzzyRound(double dVal);
	void				InsertStringToRule(LPCTSTR string);
	void				SetCombo(CComboBox* pCombo, BOOL bWhich);
	void				SetAutorun(BOOL bAutorun, int iAutorunID, int iPlayerID);
	BOOL				IsAutorun(void) const { return m_bAutorun; };
	void				AutorunAgentReady();
	void				AutorunReceivedTerminate();
	void				AutorunSendResults();
	void				Terminate();
	void				SetConnected();
	void				OnConnectionTimeout();
	void				OnConnectionError();
	void				OnActionTimer();
	void				SetNoGraphics(BOOL bVal) { if (!m_bNoGraphics) OnViewNographics(); };
	void				KickSocket();
	void				SetNoise(double noise);
	void				SetIsCoach(BOOL isOnline);

	// for blocksworld
	BOOL			GetRunning();
	BOOL			IsBlocksworld();
	BOOL			PerformCycle(TNetworkParams* params, int iNoOfCycles);
	void			StopAgent();
	void			ResetNetwork();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBasisDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void DeleteContents();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CBasisDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CBasisDoc)
	afx_msg void OnConnect();
	afx_msg void OnDisconnect();
	afx_msg void OnUpdateDisconnect(CCmdUI* pCmdUI);
	afx_msg void OnUpdateConnect(CCmdUI* pCmdUI);
	afx_msg void OnViewNographics();
	afx_msg void OnUpdateViewNographics(CCmdUI* pCmdUI);
	afx_msg void OnTeamLeft();
	afx_msg void OnUpdateTeamLeft(CCmdUI* pCmdUI);
	afx_msg void OnTeamRight();
	afx_msg void OnUpdateTeamRight(CCmdUI* pCmdUI);
	afx_msg void OnKnowledgeDorer();
	afx_msg void OnUpdateKnowledgeDorer(CCmdUI* pCmdUI);
	afx_msg void OnKnowledgeMaes();
	afx_msg void OnUpdateKnowledgeMaes(CCmdUI* pCmdUI);
	afx_msg void OnGoalTrackingOn();
	afx_msg void OnUpdateGoalTrackingOn(CCmdUI* pCmdUI);
	afx_msg void OnUseFuzzyValues();
	afx_msg void OnUpdateUseFuzzyValues(CCmdUI* pCmdUI);
	afx_msg void OnAdaptionOn();
	afx_msg void OnUpdateAdaptionOn(CCmdUI* pCmdUI);
	afx_msg void OnKnowledgeDorer2();
	afx_msg void OnUpdateKnowledgeDorer2(CCmdUI* pCmdUI);
	afx_msg void OnStartAgent();
	afx_msg void OnUpdateStartAgent(CCmdUI* pCmdUI);
	afx_msg void OnStopAgent();
	afx_msg void OnUpdateStopAgent(CCmdUI* pCmdUI);
	afx_msg void OnConnectTo();
	afx_msg void OnUpdateConnectTo(CCmdUI* pCmdUI);
	afx_msg void OnReconnect();
	afx_msg void OnUpdateReconnect(CCmdUI* pCmdUI);
	afx_msg void OnReconnectTo();
	afx_msg void OnUpdateReconnectTo(CCmdUI* pCmdUI);
	afx_msg void OnViewGenetics();
	afx_msg void OnUpdateViewGenetics(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.


#endif // !defined(AFX_BASISDOC_H__54CD01CF_4540_11D1_B242_0000C09B5CBE__INCLUDED_)
