// TAction.cpp: implementation of the TAction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TAction.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TAction::TAction(LPCTSTR name, int index, CObject* pObj, PCallBackAction pFn)
{
	SetAction(name, index, pObj, pFn);
	m_dMinActivation = 1000.0;
	m_dMaxActivation = 0.0;
	m_dMeanActivation = 0.0;
	m_dVarianz = 0.0;
	m_lNoOfActionPerformed = 0L;
	m_dDecisiveness = 0.0;
}

TAction::~TAction()
{
}

/*-----------------------------------------------------------------------------
Function: void TAction::SetAction(LPCTSTR name, CObject* pObj, PCallBackAction pFn)
Date    : 22.02.99
-------------------------------------------------------------------------------
Parameter: name of the behavior
			  pointer to the object containing the action function
			  pointer to the action function
Returnval: (void)
Description: sets the parameters of a behavior
-----------------------------------------------------------------------------*/
void TAction::SetAction(LPCTSTR name, int index, CObject* pObj, PCallBackAction pFn)
{
	m_strName = name;
	m_iIndex = index;
	m_pObject = pObj;
	m_pAction = pFn;
}

/*-----------------------------------------------------------------------------
Function: void TAction::PerformAction(void)
Date    : 22.02.99
-------------------------------------------------------------------------------
Parameter: modules activation/executability
Returnval: (void)
Description: executes the action by calling the action function
-----------------------------------------------------------------------------*/
void TAction::PerformAction(double activation, double defaultActivation)
{
	double	dNewMean;
	double	dNewN = (double) (m_lNoOfActionPerformed + 1L);
	double	dDeltaMean;
	double	dStdDev;
	double	dLeft;
	double	dRight;

	// Version with mean and stddev
	// mean activation (incrementally calculated)
	dNewMean = m_dMeanActivation + (activation - m_dMeanActivation) / dNewN;

	// standard deviation of activation (incrementally calculated)
	// Sneu = (n+1)(Mneu - M)2 + (n-1)S / n
	if (m_lNoOfActionPerformed > 0L)
	{
		dDeltaMean = dNewMean - m_dMeanActivation;
		m_dVarianz = dNewN * (dDeltaMean * dDeltaMean) 
										+ ((double)(m_lNoOfActionPerformed - 1L)
										* m_dVarianz / (double) m_lNoOfActionPerformed);
	}
	m_dMeanActivation = dNewMean;
	dStdDev = sqrt(m_dVarianz);
	dLeft = m_dMeanActivation - dStdDev;
	dRight = m_dMeanActivation + dStdDev;
	// mean - stddev is mapped to 0, mean + stddev to 1 
	if (dRight - dLeft < D_ZERO_P)
		m_dDecisiveness = 1.0;
	else if (activation < dLeft)
		m_dDecisiveness = 0.0;
	else if (activation > dRight)
		m_dDecisiveness = 1.0;
	else
		m_dDecisiveness = (activation - dLeft) / (dRight - dLeft);

/*	// Version with min and max
	// store the minimal and maximal activation with which this action is called
	if (activation < m_dMinActivation)
		m_dMinActivation = activation;
	if (activation > m_dMaxActivation)
		m_dMaxActivation = activation;

	// the activation is mapped into a range from 0.0 to 1.0 where
	// 0.0 is minimal activation, 1.0 is maximal activation
	if (m_dMaxActivation - m_dMinActivation < D_ZERO_P)
		// min and max are equal, so take max
		activation = 1.0;
	else
		activation = (activation - m_dMinActivation) / (m_dMaxActivation - m_dMinActivation);

*/
	ASSERT((m_dDecisiveness > -0.01) && (m_dDecisiveness < 1.01));

	#ifdef NO_ACTION_PARAMETER
		m_dDecisiveness = defaultActivation;
	#endif


	if ((m_pObject != NULL) && (m_pAction != NULL))
		(m_pObject->*m_pAction)(activation);
//		(m_pObject->*m_pAction)(m_dDecisiveness);

	m_lNoOfActionPerformed++;
}
