// TCompetence.h: interface for the TCompetence class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TCOMPETENCE_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TCOMPETENCE_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TEnergySpreader.h"

class TPercept;
class TAct;
class TAction;
class TProposition;
class TGoal;
class TConnection;
class TNetworkParams;
class TParseString;
class TWorldState;

/*-----------------------------------------------------------------------------
Class   : TCompetence
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: TEnergySpreader

Description: represents a competence module, a module of level 2
-----------------------------------------------------------------------------*/

class TCompetence : public TEnergySpreader  
{
	public:
		TCompetence(TNetworkParams* params, LPCTSTR name, int iNoOfRules);
		virtual ~TCompetence();

		// functions for different algorithms 		
		virtual void	CalculateActivation(int iNoOfModules, double dSumOfActivation) {};
		virtual void	CalculateExecutability() {};
		virtual void	CalculateSpreading() {};
		virtual double	GetActivationAndExecutability() { return 0.0; };
		virtual double	GetTransferedActivation() { return m_dTransferedActivation; };
		virtual void	DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen);
		virtual COLORREF	GetNameBackgroundColor();
		virtual void	ExecuteModule(double activity);
		virtual BOOL	CheckForExecution(double dThreshold, BOOL bBlocksworld);
		virtual double	Transfer(double dVal) { return dVal; };
		virtual double	SetToNewActivation();

		virtual void	ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList);
		virtual void	AddPropositionLink(TProposition* source, TProposition* dest) {};
		virtual void	AddGoalLink(TProposition* source, TProposition* prop) {};
		virtual void	AddProtectedGoalLink(TProposition* source, TProposition* prop) {};
		virtual void	AddSuccessor(TProposition* source, TProposition* dest) {};
		virtual void	AddPredecessor(TProposition* source, TProposition* dest) {};
		virtual void	AddConflictor(TProposition* source, TProposition* dest) {};

		// public methods
		BOOL		ParseRule(LPCTSTR rule, TAct* pAct, TPercept* pPercept);
		void		SetNetworkParams(TNetworkParams* params) { m_pNetworkParams = params; };
		void		ResetActivation() { SetActivation(0.0); };
		
		CString	ToString();
		
		// for blocksworld
		void		SortLinks();
		void		ExecuteBlocksworldModule();
		BOOL		Instantiate(CStringList &objects, CStringList &variables, CStringList& conditions
									, CString action, CStringList& effects, TPercept* pPercept);

		// attribute methods
		CObList*	GetPreconditions() { return m_pPreconditions; };
		CObList*	GetEffects() { return m_pEffects; };
		CObList* GetConflictors() { return m_pLinks[CONFLICTOR_CONNECTION]; };
		double	GetExecutability() { return m_dExecutability; };
		double 	GetExternActivation() { return m_dExternActivation; };
		CString	GetResults() const;
		CString	GetResultsInitial() const;
		TAction* GetAction() { return m_pAction; };
		void		SetExecuted(BOOL exec = TRUE) { m_bExecuted = exec; };
		BOOL		GetExecuted() const { return m_bExecuted; };

	protected:
		void		AddEnergySource(TConnection* connection, TProposition* srcProp, int type);
		TWorldState*	GetState(LPCTSTR prop, TPercept* pPercept);

		double			m_dExecutability;		// executability of the module
		double			m_dExternActivation;		// activation from situation and goals and previous
		double			m_dSpreadingActivation;	// activation after the spreading
		int				m_iNoOfSources;

		BOOL				m_bExecuted;

		CObList*			m_pLinks[NO_OF_CONNECTION_TYPES]; // links of the network
		CObList*			m_pPreconditions;		// (TProposition) indices and importances of the preconditions
		CObList*			m_pEffects;
		TAction*			m_pAction;				

		// for activation function
		double			m_dTransferedActivation;
		double			m_dSigmaCompetence;
		double			m_dGainCompetence;

		// pointers for easier access

		TNetworkParams*	m_pNetworkParams;


	private:
		// for Blocksworld
		BOOL		DecodeNegated(CString &str);
		BOOL		DecodeProposition(TParseString &prop, CStringList &objs, CStringList &vars);

};

#endif // !defined(AFX_TCOMPETENCE_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
