// TConnection.cpp: implementation of the TConnection class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TNetworkParams.h"
#include "TConnection.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TConnection::TConnection()
{
	m_pNetworkParams = NULL;
	m_pSourceProp = NULL;
	m_pDestinationProp = NULL;
	m_dActivation = 0.0;
	m_dMaxActivation = 1.0;
}

TConnection::TConnection(TNetworkParams* params, TProposition* src, TProposition* dest)
{
	m_pNetworkParams = params;
	m_pSourceProp = src;
	m_pDestinationProp = dest;
	m_dActivation = 0.0;
	m_dMaxActivation = 1.0;
}

TConnection::~TConnection()
{
}

/*-----------------------------------------------------------------------------
Function: void TConnection::DrawConnection(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen factors
Returnval: (void)
Description: draws a connection between two modules. It is assumed that the
				 screen coordinates of both modules have been calculated
-----------------------------------------------------------------------------*/
void TConnection::DrawConnection(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	// draw module as circle
	double	dActivation; 
	COLORREF cCol;
	CPen		pen;
	CPen		*oldPen;
	int		red;
	int		blue;

	// check, if this link has to be drawn
	if (!IsDrawn(pScreen))
		return;

	// links with zero activity are not drawn
	if (!pScreen->GetDrawZeroLinks() && (m_dActivation < 0.001) && (m_dActivation > -0.001))
		return;

	// set the maximal amount of activation this link may have
	SetMaxActivation();
	
	// set the color red = active, blue = negative active
	dActivation = m_dActivation / m_dMaxActivation;
	red = (int) (dActivation * 220.0);
	blue = 0;
	if (red > 255)
		red = 255;
	else if (red < 0)
	{
		blue = -red;
		if (blue > 255)
			blue = 255;
		red = 0;
	}
	cCol = RGB(red, 0, blue);

	// create the pen
	DoCreatePen(pen, cCol);
	oldPen = pDC->SelectObject(&pen);

	// Draw the link (which is different for different types of links
	DrawLink(pDC, pScreen);
	
	pDC->SelectObject(oldPen);
	pen.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TConnection::DrawExternalLink(CDC* pDC)
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
Returnval: (void)
Description: draws a straight line from the source module to a goal or perception
-----------------------------------------------------------------------------*/
void TConnection::DrawExternalLink(CDC* pDC)
{
	// draws a link from a goal or perception to a module
	pDC->MoveTo(m_pSourceProp->GetModule()->GetScreenCoordinates());
	pDC->LineTo(m_pDestinationProp->GetModule()->GetScreenCoordinates());
}

/*-----------------------------------------------------------------------------
Function: void TConnection::DrawInternalLink(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the drawing flags and screen information
Returnval: (void)
Description: draws an internal link that is a link from one module to another as
				 an arc
-----------------------------------------------------------------------------*/
void TConnection::DrawInternalLink(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	CPoint	start(m_pSourceProp->GetModule()->GetScreenCoordinates());
	CPoint	end(m_pDestinationProp->GetModule()->GetScreenCoordinates());
	CPoint	swap;
	int		bending;
	BOOL		bLeftToRight = TRUE;
	CRect		frame;
	int		radius = (int) (pScreen->GetScale() * (double) MODULE_SIZE);

	// the arc is bend the more the distance is
	bending = GetBending();
	
	// the starting point has to be left of the end point
	if (start.x > end.x)
	{
		swap = start;
		start = end;
		end = swap;
		bending = -bending;
		bLeftToRight = FALSE;
	}

	frame = CRect(start.x, start.y - bending, end.x, end.y +bending);
	if (bLeftToRight)
		// it is a connection from a left to a right node
		pDC->Arc(frame, start + CPoint(radius, radius), end + CPoint(-radius, radius));
	else
		// it is a connection right to left
		pDC->Arc(frame, end + CPoint(-radius, -radius), start + CPoint(radius, -radius));

	// draw the activations of the connection
	if (pScreen->GetDrawConnectionActivations())
		DrawConnectionActivations(pDC, pScreen, start, end, bending, bLeftToRight);
}

/*-----------------------------------------------------------------------------
Function: void TConnection::DrawConnectionActivations(CDC* pDC, TScreenParamsNetwork* pScreen
														  ,CPoint& start, CPoint& end, BOOL bLeftToRight)
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: left starting point of the arc, which has to be labeled
			  right endpoint
Returnval: (void)
Description: draws the activity of this connection at the middle of the arc
-----------------------------------------------------------------------------*/
void TConnection::DrawConnectionActivations(CDC* pDC, TScreenParamsNetwork* pScreen
														  , CPoint& start, CPoint& end, int bending
														  , BOOL bLeftToRight)
{
	int		nPointSize;
	CFont		font;
	CFont*	oldFont;
	CRect		field;
	COLORREF	oldTextColor;
	COLORREF	oldBkColor;
	int		sizeX;
	int		sizeY;
	int		x;
	int		y;
	CString	strActivation;

	strActivation.Format("%4.3f", m_dActivation);

	// create the font
	nPointSize = (int) (30.0 * pScreen->GetScale());
	if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
		font.CreateStockObject(SYSTEM_FONT);
	oldFont = pDC->SelectObject(&font);

	// calculate the rectangle of the text
	sizeX = (int) pScreen->GetScale() * strActivation.GetLength() * nPointSize / 30;
	sizeY = nPointSize / 5;
	x = start.x + (end.x - start.x) / 2 - sizeX / 2;
	if (bLeftToRight)
		y = end.y + bending - 7;
	else
		y = end.y + bending - sizeY + 7;

	field.SetRect(x, y, x + sizeX, y + sizeY);
	
	// color
	oldBkColor = pDC->SetBkColor(0x00ffffff);
	oldTextColor = pDC->SetTextColor(0x00000000);
	
	// draw the text
	pDC->DrawText(strActivation, field, DT_SINGLELINE | DT_VCENTER);

	pDC->SetTextColor(oldTextColor);
	pDC->SetBkColor(oldBkColor);

	pDC->SelectObject(oldFont);
	font.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: int TConnection::GetBending()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: returns the bending of an internal link
-----------------------------------------------------------------------------*/
int TConnection::GetBending()
{
	int	iDeltaIndex = m_pDestinationProp->GetModule()->GetIndex() 
							- m_pSourceProp->GetModule()->GetIndex();
	if (iDeltaIndex < 0)
		iDeltaIndex = -iDeltaIndex;
	return iDeltaIndex * 10;
}

/*-----------------------------------------------------------------------------
Function: TEnergySpreader* TConnection::GetSourceModule()
Date    : 20.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: the module which is the source of this connection
Description: _
-----------------------------------------------------------------------------*/
TEnergySpreader* TConnection::GetSourceModule()
{
	return m_pSourceProp->GetModule();
}

/*-----------------------------------------------------------------------------
Function: TEnergySpreader* TConnection::GetDestinationModule()
Date    : 20.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: the module which is the destination of this connection
Description: _
-----------------------------------------------------------------------------*/
TEnergySpreader* TConnection::GetDestinationModule()
{
	return m_pDestinationProp->GetModule();
}

//--------------------------------------------------------------------
// Maes
//--------------------------------------------------------------------

