// TDorer2Competence.h: interface for the TDorer2Competence class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TDorer2Competence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TDorer2Competence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TCompetence.h"

class TGoal;
class TConnection;
class TNetworkParams;
class TDorer2BaseGoalTrack;
class TScreenParamsNetwork;
class TResourceFactory;

/*-----------------------------------------------------------------------------
Class   : TDorer2Competence
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: TEnergySpreader

Description: represents a competence module, a module of level 2
-----------------------------------------------------------------------------*/

class TDorer2Competence : public TCompetence  
{
	public:
		TDorer2Competence(TNetworkParams* params, LPCTSTR name, int iNoOfRules, int iNoOfGoals);
		virtual ~TDorer2Competence();

		// functions for different algorithms (haesslich!!)
		
		virtual void	CalculateActivation(int iNoOfModules = 0, double dSumOfActivation = 0.0);
		virtual void	CalculateExecutability();
		virtual double	GetActivationAndExecutability();
		virtual BOOL	CheckForExecution(double dThreshold, BOOL bBlocksworld);
		virtual double	Transfer(double dVal);
		virtual double	SetToNewActivation();
		virtual void	DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen);
		
		virtual void	ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList);
		virtual void	AddPropositionLink(TProposition* source, TProposition* dest);
		virtual void	AddGoalLink(TProposition* source, TProposition* prop);
		virtual void	AddProtectedGoalLink(TProposition* source, TProposition* prop);
		virtual void	AddSuccessor(TProposition* source, TProposition* dest);
		virtual void	AddPredecessor(TProposition* source, TProposition* dest);
		virtual void	AddConflictor(TProposition* source, TProposition* dest);
		virtual void	AddResourceLink(TProposition* source, TProposition* dest);

		BOOL		ParseRule(LPCTSTR rule, TAct* pAct, TPercept* pPercept, TResourceFactory* pResourceFactory);	
		double	GetTransferedActivation(int iGoalIndex);
		void		UpdateResourceUsage();
		double	GetSumTaup() const { return m_dSumTaup; };

	protected:

		double	CalculatePreviousActivation();
		void		CalculateExternActivation();
		void		CalculateSpreadingActivation();

	private:
		TDorer2BaseGoalTrack*	m_pGoalTrack;
		CObList*						m_pResources;
		CObList*						m_pResourceLinks;
		double						m_dSumTaup;
};

#endif // !defined(AFX_TDorer2Competence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
