// TDorerCompetence.cpp: implementation of the TDorerCompetence class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TWorldState.h"
#include "TGoal.h"
#include "TGoalTrack.h"
#include "TNetworkParams.h"
#include "TDorerConnection.h"
#include "TDorerCompetence.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorerCompetence::TDorerCompetence(TNetworkParams* params, LPCTSTR name, int index, int iNoOfGoals)
					:TCompetence(params, name, index)
{
	m_pGoalTrack = new TGoalTrack(iNoOfGoals);
	m_bGoalTrackingOn = FALSE;

	SetMaxActivation(MAX_ACTIVATION_SIGMOID);
	SetAdaptActivationAreaOn(FALSE);
}

TDorerCompetence::~TDorerCompetence()
{
	delete m_pGoalTrack;
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddPropositionLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the proposition that is a precondition of this module
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddPropositionLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorerSituationLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, SITUATION_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddGoalLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddGoalLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorerGoalLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, GOAL_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddProtectedGoalLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddProtectedGoalLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorerProtectedGoalLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, PROTECTED_GOAL_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddSuccessor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddSuccessor(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorerSuccessorLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, SUCCESSOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddPredecessor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddPredecessor(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorerPredecessorLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, PREDECESSOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::AddConflictor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the proposition (effect) of the passed module
			  pointer to the proposition (precondition) of this module
Returnval: (void)
Description: adds an energysource of which this module is a conflictor
-----------------------------------------------------------------------------*/
void TDorerCompetence::AddConflictor(TProposition* source, TProposition* dest)
{
	// add the module
	TConnection* newConnection = new TDorerConflictorLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, CONFLICTOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::SetAdaptActivationAreaOn(BOOL bVal)
Date    : 05.10.98
-------------------------------------------------------------------------------
Parameter: flag indicating, whether there should be activation adaption (TRUE) or not
Returnval: (void)
Description: switches on or off the adaption of the activation area (aaa). aaa means
				 the sigmoidal activation function is adapted from a small ranged steep
				 curve to a broader one if necessary. Each time activation exceeds the
				 y-coordinates value of one of the two control points for the sigmoid,
				 the x-coordinate is shifted to the right (left) in case of the right
				 (left) control point. This should realize, that each module is within an
				 optimal activation state on the almost linear part of the sigmoidal curve.
-----------------------------------------------------------------------------*/
void TDorerCompetence::SetAdaptActivationAreaOn(BOOL bVal)
{
	m_bAdaptActivationAreaOn = bVal;

	if (bVal)
	{
		// adaption on
		// Version 15.10.98: translation of activation function to left and right
		m_dSigmaCompetence = m_pNetworkParams->GetSigma();
		m_dGainCompetence = m_pNetworkParams->GetGain();
		// first point on the activatoin function
		m_dPointOnSigmoidal1Y = 0.1;
		m_dPointOnSigmoidal1X = m_dSigmaCompetence 
										- log(1.0 / m_dPointOnSigmoidal1Y - 1.0) / m_dGainCompetence;
		m_dPointOnSigmoidal2Y = 0.8;
		m_dPointOnSigmoidal2X = m_dSigmaCompetence 
										- log(1.0 / m_dPointOnSigmoidal2Y - 1.0) / m_dGainCompetence;

		// Variation 5.10.98
		// the activation area is small in the beginning (0.4, 0.1) - (0.6, 0.9)
		// and then enlarged if less than 0.1 or more than 0.9 activation is spread
/*		m_dPointOnSigmoidal1X = 0.4;
		m_dPointOnSigmoidal1Y = 0.1;
		m_dPointOnSigmoidal2X = 0.6;	
		m_dPointOnSigmoidal2Y = 0.9;

*/		RecalculateGainAndSigma();	
	}
	else
	{
		// no adaption, so take gain and sigma as preset
		m_dSigmaCompetence = m_pNetworkParams->GetSigma();
		m_dGainCompetence = m_pNetworkParams->GetGain();
	}
}
/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters (translation, scaling, ...)
			  maximal reachable activation (for color scaling)
Returnval: (void)
Description: draws the connections of the competence module and the module itself
-----------------------------------------------------------------------------*/
void TDorerCompetence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	// draw the module itself
	TCompetence::DrawModule(pDC, pScreen);

	// draw goal tracking
	if (pScreen->GetDrawGoalTracking() && m_bGoalTrackingOn)
		m_pGoalTrack->Draw(pDC, pScreen, m_tScreenCoord);
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::CalculateActivation(int iNoOfModules, double dSumOfActivation)
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the class containing the network parameters such as phi, gamma, ...
Returnval: (void)
Description: first the old activation is decayed
				 calculates the activation of this module. This is done by summing
				 up the energy of each incoming connection to this module according
				 to Maes. 
-----------------------------------------------------------------------------*/
void TDorerCompetence::CalculateActivation(int iNoOfModules, double dSumOfActivation)
{
	double			dPreviousActivation;
	double			dExternActivation;
	double			dSpreadingActivation;

	// previous activation
	dPreviousActivation = CalculatePreviousActivation();

	// extern activation 
	dExternActivation = CalculateExternActivation();

	if (m_bGoalTrackingOn)
	{
		// tell goal tracking the absolute previous and extern (allways positive) influence
		if (dPreviousActivation < 0.0)
			m_pGoalTrack->SetAbsActivation(dExternActivation - dPreviousActivation);
		else
			m_pGoalTrack->SetAbsActivation(dExternActivation + dPreviousActivation);
	}

	// spreading activation
	dSpreadingActivation = CalculateSpreadingActivation();

	// and sum them up (variables used are for compatibility with Maes spreading)
	m_dSpreadingActivation = dPreviousActivation + dExternActivation + dSpreadingActivation;
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::CalculatePreviousActivation()
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: activation of the module, that's source is the previous activation
			  of the module
Description: does the decay of activation
-----------------------------------------------------------------------------*/
double TDorerCompetence::CalculatePreviousActivation()
{
	double	beta = m_pNetworkParams->GetBeta();
	double	dActivation = m_dMainActivation * beta;

	// take activation of previous spreading into account
	// decay goal track
	if (m_bGoalTrackingOn)
		m_pGoalTrack->ReduceActivation(beta);

	return dActivation;
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::CalculateExternActivation()
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: activation received from perceptions and goals. Goals are not included
			  if goaltracking is on
Description: receives the extern activation from goals and perception and does
				 goaltracking
-----------------------------------------------------------------------------*/
double TDorerCompetence::CalculateExternActivation()
{
	POSITION			pos;
	TConnection*	pCon;
	int				i;
	double			dSum;
	int				n;
	double			activation;

	// situation connections
	dSum = 0.0;
	activation = 0.0;
	pos = m_pLinks[SITUATION_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[SITUATION_CONNECTION]->GetNext(pos);
		activation += pCon->GetActivation();
	}

	// the activation is divided by the number of sources
	n = m_pLinks[SITUATION_CONNECTION]->GetCount();
	if (n > 1)
		activation /= (double) n;
	dSum += activation;


	// goal connections
	pos = m_pLinks[GOAL_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[GOAL_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();
		
		// do goal tracking
		if (m_bGoalTrackingOn)
		{
			i = pCon->GetSourceModule()->GetIndex();
			m_pGoalTrack->SetActivation(i, activation);
		}
		else
			dSum += activation;
	}

	
	// protected goal connections
	pos = m_pLinks[PROTECTED_GOAL_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[PROTECTED_GOAL_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();

		// do goal tracking
		if (m_bGoalTrackingOn)
		{
			i = pCon->GetSourceModule()->GetIndex();
			m_pGoalTrack->SetActivation(i, activation);
		}
		else
			dSum += activation;
	}
	
	return dSum;
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::CalculateSpreadingActivation()
Date    : 28.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: activation spread to this module by other modules
			  if goal tracking is on, the influence of direct goals may be within this
Description: calculates the new activation that is gained by activation spreading
				 among the modules
-----------------------------------------------------------------------------*/
double TDorerCompetence::CalculateSpreadingActivation()
{
	POSITION			pos;
	TConnection*	pCon;
	double			n;
	double			activation;
	TGoalTrack		goalTrack(m_pGoalTrack->GetNoOfGoals());
	double			dSum;

	// predecessors
	dSum = 0.0;
	n = (double) m_pLinks[PREDECESSOR_CONNECTION]->GetCount();
	pos = m_pLinks[PREDECESSOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TConnection*) m_pLinks[PREDECESSOR_CONNECTION]->GetNext(pos);
		// divide by the number of connections
		activation = pCon->GetActivation() / n;
		// goal track is in percentage, recalculate absolute activations
		// and get the activation, that is reduced by the goals activation
		if (m_bGoalTrackingOn)
		{
			goalTrack.SetGoalTrack(((TDorerCompetence*) pCon->GetSourceProp()->GetModule())->GetGoalTrack());
			activation = goalTrack.SetAbsActivationFromPercentage(activation);
			m_pGoalTrack->UpdateActivation(&goalTrack);
		}

		dSum += activation;
	}

	// successors
	pos = m_pLinks[SUCCESSOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TConnection*) m_pLinks[SUCCESSOR_CONNECTION]->GetNext(pos);
		// get the activation and goal track of this successor
		activation = pCon->GetActivation();

		// goal track is in percentage, recalculate absolute activations
		// and get the activation, that is reduced by the goals activation
		if (m_bGoalTrackingOn)
		{
			goalTrack.SetGoalTrack(((TDorerCompetence*) pCon->GetSourceProp()->GetModule())->GetGoalTrack());
			activation = goalTrack.SetAbsActivationFromPercentage(activation);
			m_pGoalTrack->UpdateActivation(&goalTrack);
		}

		dSum += activation;
	}


	// conflictors
	pos = m_pLinks[CONFLICTOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TConnection*) m_pLinks[CONFLICTOR_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();

		// goal track is in percentage, recalculate absolute activations
		// and get the activation, that is reduced by the goals activation
		if (m_bGoalTrackingOn)
		{
			goalTrack.SetGoalTrack(((TDorerCompetence*) pCon->GetSourceProp()->GetModule())->GetGoalTrack());
			activation = goalTrack.SetAbsActivationFromPercentage(activation);
			m_pGoalTrack->UpdateActivation(&goalTrack);
		}
		
		dSum += activation;
	}

	// the influence of the goal tracks has to be added now
	if (m_bGoalTrackingOn)
	{
		activation = m_pGoalTrack->GetTrackedActivation();
		dSum += activation;
	}
	
	return dSum;
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::CalculateExecutability()
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates the executability of this module. This is done by multiplying
				 the value of each precondition. The result is powered to 1/number of 
				 preconditions
-----------------------------------------------------------------------------*/
void TDorerCompetence::CalculateExecutability()
{
	POSITION		pos;
	int			n;
	double		value;

	n = 0;
	m_dExecutability = 1.0;
	pos = m_pPreconditions->GetHeadPosition();
	// step through each precondition
	while (pos != NULL)
	{
		value = ((TProposition*) m_pPreconditions->GetNext(pos))->GetValue();
		#ifdef FUZZY_MIN_AND
			// this version uses min as fuzzy and	
			if (value < m_dExecutability)
				m_dExecutability = value;
		#else 
			// this version uses product as fuzzy min
			m_dExecutability *= value;
		#endif
		n++;
	}

	// the executability is powered by 1 / number of preconditions
	#ifdef FUZZY_SQRT_AND
		if (n > 2)
			m_dExecutability = pow(m_dExecutability, 1.0 / (double) n);
		else if (n == 2)
			m_dExecutability = sqrt(m_dExecutability);
	#endif
}


/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::SetToNewActivation() 
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: new activation of the module
Description: sets the activation to the previously calculated new activation
				 it is neccessary to first calculate all new activations of each
				 module and then set it to the new value, because otherwise the
				 algorithm would depend on the order of calculation
-----------------------------------------------------------------------------*/
double TDorerCompetence::SetToNewActivation() 
{ 
	m_dTransferedActivation = Transfer(m_dSpreadingActivation);
	if (m_bAdaptActivationAreaOn)
	{
		// is activation fairly big (> 0.9) ?
		if (m_dTransferedActivation > m_dPointOnSigmoidal2Y)
		{
			// move activation area to the right
			m_dPointOnSigmoidal2X += 0.02;
			// Version 15.10.98: translation of activation function to left and right
			m_dPointOnSigmoidal1X += 0.02;
			RecalculateGainAndSigma();
		}
		// is activation fairly big (> 0.9) ?
		else if (m_dTransferedActivation < m_dPointOnSigmoidal1Y)
		{
			// move activation area to the left
			m_dPointOnSigmoidal1X -= 0.02;
			// Version 15.10.98: translation of activation function to left and right
			m_dPointOnSigmoidal2X -= 0.02;
			RecalculateGainAndSigma();
		}
	}

	return (SetActivation(m_dSpreadingActivation));
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::Transfer(double dVal)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: activation of a module
Returnval: sigmoid of the value
Description: calculates the transfer function of a modules activity (the activity
				 that is spread). Current transfer function is a sigmoid function
				 (see Goetz 1997)
-----------------------------------------------------------------------------*/
double TDorerCompetence::Transfer(double dVal)
{
	// 1 / 1 + e ^ (m * (s - x))
//	return 1.0 / (1.0 + exp(m_pNetworkParams->GetGain() * (m_pNetworkParams->GetSigma() - dVal)));
	return 1.0 / (1.0 + exp(m_dGainCompetence * (m_dSigmaCompetence - dVal)));
}

/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::RecalculateGainAndSigma()
Date    : 01.10.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates gain and sigma of the sigmoidal transfer function. They
				 are determined by the two points m_dPointOnSigmoidal1 and 2, which 
				 have to lie on the function
-----------------------------------------------------------------------------*/
void TDorerCompetence::RecalculateGainAndSigma()
{
	double	temp = log(1.0 / m_dPointOnSigmoidal1Y - 1.0);

	m_dGainCompetence = (log(1.0 / m_dPointOnSigmoidal2Y - 1.0) - temp)
							/ (m_dPointOnSigmoidal1X - m_dPointOnSigmoidal2X);
	m_dSigmaCompetence = temp / m_dGainCompetence + m_dPointOnSigmoidal1X;
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::GetActivationAndExecutability()
Date    : 18.11.98
-------------------------------------------------------------------------------
Parameter: activation of the module
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
double TDorerCompetence::GetActivationAndExecutability()
{
	// Dorer uses Product
	return m_dExecutability * GetActivation();
}