// TEnergySpreader.h: interface for the TEnergySpreader class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TENERGYSPREADER_H__76B7A461_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TENERGYSPREADER_H__76B7A461_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


class TConnectionList;
class TProposition;
class TScreenParamsNetwork;

/*-----------------------------------------------------------------------------
Class   : TEnergySpreader
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: CObject

Description: Base class for all modules spreading energy to other modules
-----------------------------------------------------------------------------*/

class TEnergySpreader : public CObject  
{
	public:
		TEnergySpreader();
		TEnergySpreader(LPCTSTR name, int index);
		virtual ~TEnergySpreader();

		void		Initialize();

		// energy functions
		virtual double		GetActivation() { return m_dMainActivation; };
		double				SetActivation(double activation);
	
		virtual void		DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen);
		virtual COLORREF	GetNameBackgroundColor() { return 0x00ffffff; };
		void					SetNormCoordinates(CPoint point) { m_tNormCoord = point; };
		void					SetCoordinates(TScreenParamsNetwork* pScreen);
		CPoint				GetScreenCoordinates() const { return m_tScreenCoord; };
		BOOL					IsPointWithinModule(CPoint point, double size);
		void					WriteActivationHistory(CArchive& ar);
		void					WriteActivationDistribution(CArchive& ar);

		// notification
		void		OnAddedToDestination() { m_iNoOfDestinations++; };
		void		OnRemovedFromDestination() { m_iNoOfDestinations--; };

		// get and set functions
		virtual double 	GetMainActivation() const { return m_dMainActivation; };
		double	GetMaxActivation() const { return m_dMaxActivation; };
		void		SetMaxActivation(double activation) { m_dMaxActivation = activation; };
		CString	GetName() { return m_strName; };
		void		SetName(CString &str) { m_strName = str; };
		int		GetIndex() const { return m_iIndex; };
		void		SetIndex(int index) { m_iIndex = index; };
		BOOL		IsSelected(void) const { return m_bSelected; };
		void		SetSelected(BOOL sel = TRUE) { m_bSelected = sel; };

	protected:

		CString	m_strName;
		int		m_iIndex;
		double	m_dMainActivation;
		double	m_dActivationChange;
		double	m_dMaxActivation;
		int		m_iNoOfDestinations;
		CPoint	m_tNormCoord;					// normalized  coordinate
		CPoint	m_tScreenCoord;				// screen coordinates
		BOOL		m_bSelected;					// true, if this module is the selected one

	private:
		void		DrawName(CDC* pDC, TScreenParamsNetwork* pScreen);

		#ifdef ACTIVATION_HISTORY
			double	m_adHistory[MAX_HISTORY];
			int		m_iNoOfHistory;
		#endif

		#ifdef ACTIVATION_DISTRIBUTION
			int		m_aiDistribution[NO_OF_SLOTS];
		#endif
};

#endif // !defined(AFX_TENERGYSPREADER_H__76B7A461_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
