// TGoalTrack.h: interface for the TGoalTrack class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TGOALTRACK_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_)
#define AFX_TGOALTRACK_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TScreenParamsNetwork;

/*-----------------------------------------------------------------------------
Class   : TGoalTrack
Date    : 07.07.98
-------------------------------------------------------------------------------
Base Class: CObject

Description: encapsules the goal tracking
-----------------------------------------------------------------------------*/

class TGoalTrack : public CObject  
{
	public:
		TGoalTrack(int iNoOfGoals);
		virtual ~TGoalTrack();

		// public methods
		void			SetGoalTrack(TGoalTrack* pSource);
		void			ReduceActivation(double beta);
		void			SetPercentage(double Activation);
		double		SetAbsActivationFromPercentage(double Activation);
		double		GetTrackedActivation(void);
		void			UpdateActivation(TGoalTrack* pSource);
		void			UpdatePercentage(double dActivation);

		void			Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords);

		// attribute functions		
		int			GetNoOfGoals(void) const { return m_iNoOfGoals; };
		double		GetSumActivation(void) const { return m_dSumActivation; };
		double		GetAbsActivation(int i) { ASSERT ((i >= 0) && (i < m_iNoOfGoals)); 
															return (m_pdActivation[i] * m_dSumActivation); };
		
		void			SetAbsActivation(double val) { m_dAbsActivation = val; };
		void			SetActivation(int i, double val); 
		double		GetActivation(int i) { ASSERT ((i >= 0) && (i < m_iNoOfGoals)); 
															return m_pdActivation[i]; };
		double		GetPercent(int i) { ASSERT ((i >= 0) && (i < m_iNoOfGoals)); 
													return m_pdPercent[i]; };

	private:
		int				m_iNoOfGoals;			// no of goals in the goal track
		double			m_dSumActivation;		// 100 % activation
		double			m_dAbsActivation;		// absolute activation added this cycle
		double*			m_pdPercent;			// relative value of goal influences
		double*			m_pdActivation;		// procentual values of goal influences
};

#endif // !defined(AFX_TGOALTRACK_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_)
