// TMaesNetwork.cpp: implementation of the TMaesNetwork class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TRuleView.h"
#include "TNetworkView.h"
#include "TMaesCompetence.h"
#include "TAct.h"
#include "TPercept.h"
#include "TNetworkParams.h"
#include "DNetworkParamsSlider.h"
#include "TMaesNetwork.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TMaesNetwork::TMaesNetwork(TNetworkView* pNView, TRuleView* pRView)
					: TBehaviorNetwork(pNView, pRView) 
{
	m_pRulesView->SetBehaviorNetwork(this);
	m_pNetworkView->SetNetwork(this);

	Initialize();
	SetUseFuzzyValues(FALSE);
	SetFuzzyRound(0.5);
}


TMaesNetwork::~TMaesNetwork()
{
}

void TMaesNetwork::Initialize()
{
	m_dActivationSum = 0.0;
	m_pNetworkParams->SetGamma(GAMMA);
	m_pNetworkParams->SetPhi(PHI);
	m_pNetworkParams->SetDelta(DELTA);
	m_pNetworkParams->SetNetworkPi(NETWORK_PI);
	m_pNetworkParams->SetTheta(THETA);
	m_pNetworkParams->SetThetaReduction(THRESHOLD_REDUCTION);

	TBehaviorNetwork::Initialize();
}

TCompetence* TMaesNetwork::CreateNewCompetence()
{
	return new TMaesCompetence(m_pNetworkParams, "", GetNoOfRules());
}

/*-----------------------------------------------------------------------------
Function: double TMaesNetwork::DoActivationSpreading()
Date    : 19.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: sum of activation in the network
Description: calculates the new input to the network and the spreading of activation
				 within the network. Only one cycle of activation spreading is executed.
				 It is assumed, that there is real perception available, that is, the
				 agent is either connected to the server, or provided with some logged
				 or fake perception
-----------------------------------------------------------------------------*/
double TMaesNetwork::DoActivationSpreading()
{
	m_dActivationSum = TBehaviorNetwork::DoActivationSpreading();
	return m_dActivationSum;
}

/*-----------------------------------------------------------------------------
Function: void TMaesNetwork::CalculateSpreading()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: does the spreading of activation for each competence module
-----------------------------------------------------------------------------*/
void TMaesNetwork::CalculateSpreading()
{
	POSITION pos = m_pCompetenceList->GetHeadPosition();
	while (pos != NULL)
		((TCompetence*) m_pCompetenceList->GetNext(pos))->CalculateSpreading();
}

/*-----------------------------------------------------------------------------
Function: void TMaesNetwork::ParamsDialog()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: allows to change the parameters of the network easily. The parameters are
				changed immediately, so if user presses Cancel afterwards, the old 
				parameters have to be saved and restored at Cancel
-----------------------------------------------------------------------------*/
void TMaesNetwork::ParamsDialog()
{
	TNetworkParams			oldParams(m_pNetworkParams);

	// check the type of the network
	DNetworkParamsSlider	tDialog(NULL, m_pNetworkParams);

	tDialog.m_dMin = MIN_MAES;
	tDialog.m_dMax = MAX_MAES;
	tDialog.m_dRange = RANGE_MAES;
	tDialog.m_dMinGamma = MIN_GAMMA;
	tDialog.m_dMaxBeta = MAX_MAES;
	tDialog.m_dMaxThetaReduction = MAX_THETA_REDUC;

	tDialog.m_dPi = m_pNetworkParams->GetNetworkPi();
	tDialog.m_dPhi = m_pNetworkParams->GetPhi();
	tDialog.m_dGamma = m_pNetworkParams->GetGamma();
	tDialog.m_dDelta = m_pNetworkParams->GetDelta();
	tDialog.m_dTheta = m_pNetworkParams->GetTheta();
	tDialog.m_dThetaReduction = m_pNetworkParams->GetThetaReduction();
	tDialog.m_dFuzzyRound = m_pNetworkParams->GetFuzzyRound();
	if (tDialog.DoModal() != IDOK)
		// restore old params, because user canceled
		m_pNetworkParams->SetNetworkParams(&oldParams);
}

double TMaesNetwork::GetActivationSum() const 
{
	return m_dActivationSum; 
}
