// TProposition.cpp: implementation of the TProposition class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TWorldState.h"
#include "TProposition.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TProposition::TProposition()
{
	m_tWeigh = 0.0;
	m_bNegated = FALSE;
	m_pWorldState = NULL;
	m_pModule = NULL;
}

TProposition::~TProposition()
{
}

void TProposition::SetProposition(TEnergySpreader* module, TWorldState* state
											 , double weigh, BOOL negated)
{
	m_pModule = module;
	m_pWorldState = state;
	m_tWeigh = weigh;
	m_bNegated = negated;
}

/*-----------------------------------------------------------------------------
Function: double TProposition::GetValue()
Date    : 28.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: value of the proposition, that is the truth of the proposition or 1-truth 
			  for negated propositions
Description: 
-----------------------------------------------------------------------------*/
double TProposition::GetValue() const
{
	if (m_bNegated)
		return (1.0 - m_pWorldState->GetMainActivation());		
	else
		return m_pWorldState->GetMainActivation();
}

void TProposition::SetValue(double val)
{
	m_pModule->SetActivation(val);		
}

/*-----------------------------------------------------------------------------
Function: CString TProposition::ToString(TPercept* percept)
Date    : 15.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the percept object needed to convert the index to a string
Returnval: this object converted to a string like [not] nameOfProposition %dvalue
Description: converts the proposition into a string
-----------------------------------------------------------------------------*/
CString TProposition::ToString()
{
	CString ret;
	CString not;

	if (m_bNegated)
		not = "not ";
	ret.Format("%s  %1.3f", m_pWorldState->GetName(), m_tWeigh);
	return not + ret;
}

/*-----------------------------------------------------------------------------
Function: void TProposition::ChangeWorldState()
Date    : 22.10.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: called to set the world state according to this proposition which means
				 if this proposition is negated, the world state gets 0.0 else it gets 1.0
-----------------------------------------------------------------------------*/
void TProposition::SetWorldState()
{
	if (m_bNegated)
		m_pWorldState->SetActivation(0.0);
	else
		m_pWorldState->SetActivation(1.0);
}
