// TResource.cpp: implementation of the TResource class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TResource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TResource::TResource(LPCTSTR name, int index, CObject* pObj, PCallBackPerception pCall, int iMaxAvail)
		: TWorldState(name, index, pObj, pCall)
{
	m_iMaxAvailable = iMaxAvail;
	ResetAmountAvailable();
}

TResource::~TResource()
{
}

/*-----------------------------------------------------------------------------
Function: void TResource::ReduceActivationLevel(double dDeltaTheta)
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: network parameter ThetaReduction
Returnval: (void)
Description: reduces the activation level of the resource by deltaTheta, but not
				 below or equal zero
-----------------------------------------------------------------------------*/
void TResource::ReduceActivationLevel(double dDeltaTheta)
{
	double	dActivation = GetActivation();

	dActivation -= dDeltaTheta;
	if (dActivation < 0.00001)
		dActivation = 0.00001;
	SetActivation(dActivation);
}

/*-----------------------------------------------------------------------------
Function: void TResource::ResetActivationLevel(double dTheta)
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: network parameter theta
Returnval: (void)
Description: resets the activation level of the resource to theta. called after
				 a module has been executed
-----------------------------------------------------------------------------*/
void TResource::ResetActivationLevel(double dTheta)
{
	SetActivation(dTheta);
}

/*-----------------------------------------------------------------------------
Function: void TResource::ReduceAmountAvailable(int iReduction)
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: amount to reduce the resources units available
Returnval: (void)
Description: reduces the amount of resource units available
-----------------------------------------------------------------------------*/
void TResource::ReduceAmountAvailable(int iReduction)
{
	ASSERT(iReduction <= m_iAmountAvailable);
	m_iAmountAvailable -= iReduction;
}

/*-----------------------------------------------------------------------------
Function: void TResource::IncreaseAmountAvailable(int iIncrease)
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: amount to increase the resources units available
Returnval: (void)
Description: increases the amount of resource units available
-----------------------------------------------------------------------------*/
void TResource::IncreaseAmountAvailable(int iIncrease)
{
	ASSERT(iIncrease + m_iAmountAvailable <= m_iMaxAvailable);
	m_iAmountAvailable += iIncrease;
}
