// TWorldState.h: interface for the TWorldState class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TWORLDSTATE_H__89CEB9A1_BF15_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TWORLDSTATE_H__89CEB9A1_BF15_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TEnergySpreader.h"

class TProposition;

/*-----------------------------------------------------------------------------
Class   : TWorldState
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: TEnergySpreader

Description: represents the propositions of the world
-----------------------------------------------------------------------------*/

class TWorldState : public TEnergySpreader  
{
	public:
		TWorldState(LPCTSTR name, int index, CObject* pObj, PCallBackPerception pCall);
		virtual ~TWorldState();

		void		Initialize();
		double	GetTruthValue();
		virtual double GetMainActivation() const;

		// notification functions
		void	OnAddedToPreconditions() { m_iNoOfOccurencesInPreconditions++; };
		void	OnAddedToInhibition() { m_iNoOfOccurencesInInhibitionLinks++; };
		void	OnAddedToExcitation() { m_iNoOfOccurencesInExcitationLinks++; };
		
		int	GetNoOfOccurencesInPreconditions() const { return m_iNoOfOccurencesInPreconditions; };
		int	GetNoOfOccurencesInInhibitionLinks() const { return m_iNoOfOccurencesInInhibitionLinks; };
		int	GetNoOfOccurencesInExcitationLinks() const { return m_iNoOfOccurencesInExcitationLinks; };
		BOOL	IsUsed(void) const { return m_bUsed; };
		void	SetUsed(BOOL used = TRUE) { m_bUsed = used; };
		void	SetFuzzy(BOOL bFuzzy = TRUE) { m_bFuzzy = bFuzzy; };
		BOOL	GetFuzzy() const { return m_bFuzzy; };
		void	SetFuzzyRound(double dRound) { m_dFuzzyRound = dRound; };
		double GetFuzzyRound() const { return m_dFuzzyRound; };
		TProposition* GetProposition() { return m_pProposition; };

	private:
		int	m_iNoOfOccurencesInPreconditions;
		int	m_iNoOfOccurencesInInhibitionLinks;	// about the same as no of delete effects
		int	m_iNoOfOccurencesInExcitationLinks;	// about the same as no of add effects

		TProposition*			m_pProposition;
		BOOL						m_bUsed;				// true, if used by current network
		CObject*					m_pObject;
		PCallBackPerception	m_pTruthFunction;
		BOOL						m_bFuzzy;
		double					m_dFuzzyRound;
};

#endif // !defined(AFX_TWORLDSTATE_H__89CEB9A1_BF15_11D1_B242_0000C09B5CBE__INCLUDED_)
