// TRuleView.cpp : implementation file
//

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TBehaviorNetwork.h"
#include "TRuleView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TRuleView

IMPLEMENT_DYNCREATE(TRuleView, CEditView)

TRuleView::TRuleView()
{
	m_strFileName.Empty();
	m_bHaveStandardFile = FALSE;
}

TRuleView::~TRuleView()
{
}


BEGIN_MESSAGE_MAP(TRuleView, CEditView)
	//{{AFX_MSG_MAP(TRuleView)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN_RULES, OnUpdateFileOpenRules)
	ON_COMMAND(ID_FILE_OPEN_RULES, OnFileOpenRules)
	ON_COMMAND(ID_FILE_SAVE_AS_RULES, OnFileSaveAsRules)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS_RULES, OnUpdateFileSaveAsRules)
	ON_COMMAND(ID_FILE_SAVE_RULES, OnFileSaveRules)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_RULES, OnUpdateFileSaveRules)
	ON_COMMAND(ID_PARSE_RULES, OnParseRules)
	ON_UPDATE_COMMAND_UI(ID_PARSE_RULES, OnUpdateParseRules)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TRuleView drawing

void TRuleView::OnDraw(CDC* pDC)
{
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// TRuleView diagnostics

#ifdef _DEBUG
void TRuleView::AssertValid() const
{
	CEditView::AssertValid();
}

void TRuleView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// TRuleView message handlers

BOOL TRuleView::OpenFile(BOOL bAskForName)
{
	CFile		file;
	CString	message;

	// ask for the file to open
	if (bAskForName)
	{
		CFileDialog fileDlg(TRUE, "*.txt", NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR
								  , "Rule Files (*.txt)|*.txt|All Files (*.*)|*.*", this);
		if (fileDlg.DoModal() != IDOK)
			return FALSE;
		m_strFileName = fileDlg.GetPathName();
	}

	// open the file and archive
	if(!file.Open(m_strFileName, CFile::modeRead))
		return FALSE;
	
	CArchive	archive(&file, CArchive::load);

	// and load it
	SerializeRaw(archive);
	archive.Close();
	file.Close();
	return TRUE;
}

void TRuleView::OnFileOpenRules() 
{
	OpenFile();
	OnParseRules();
}

void TRuleView::OnFileSaveAsRules() 
{
	// ask for the filename
	CFileDialog fileDlg(FALSE, "*.txt", m_strFileName, OFN_HIDEREADONLY | OFN_NOCHANGEDIR
							  , "Rule Files (*.txt)|*.txt|All Files (*.*)|*.*|", this);
	if (fileDlg.DoModal() != IDOK)
		return;
	m_strFileName = fileDlg.GetPathName();
	
	if (m_strFileName.GetLength() > 0)
		OnFileSaveRules();
}

void TRuleView::OnFileSaveRules() 
{
	if (m_strFileName.GetLength() == 0)
		// the file has no name
		OnFileSaveAsRules();
	else
	{
		CFile		file;
		CString	message;

		// open the file and archive
		if(!file.Open(m_strFileName, CFile::modeCreate | CFile::modeWrite))
		{
			message.Format("Unable to open file %s for writing", m_strFileName);
			AfxMessageBox(message);
			return;
		}
		
		CArchive	archive(&file, CArchive::store);

		// and save it
		SerializeRaw(archive);
		archive.Close();
		file.Close();
	}
}

void TRuleView::Parse() 
{
	OnParseRules();
}

void TRuleView::InsertString(LPCTSTR name)
{
	// inserts the passed string at the current position
	CPoint	position = GetCaretPos();
	int		iCursorPosition = GetEditCtrl().CharFromPos(position);

	GetEditCtrl().SetSel(iCursorPosition, iCursorPosition);
	GetEditCtrl().ReplaceSel(name);
	GetEditCtrl().SetSel(-1,-1);
}


void TRuleView::OnParseRules() 
{
	CString	rules;

	// get the string containing the rules
	GetEditCtrl().GetWindowText(rules);
	if (rules.Find(STRING_OBJECTS) >= 0)
		m_pKnowledge->ParseRulesWithVariables(rules);
	else
		m_pKnowledge->ParseRules(rules);
}


void TRuleView::OnUpdateFileOpenRules(CCmdUI* pCmdUI) 
{
}

void TRuleView::OnUpdateFileSaveAsRules(CCmdUI* pCmdUI) 
{
}

void TRuleView::OnUpdateFileSaveRules(CCmdUI* pCmdUI) 
{
	if (GetEditCtrl().GetModify())
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TRuleView::OnUpdateParseRules(CCmdUI* pCmdUI) 
{
//	if (GetEditCtrl().GetModify())
		pCmdUI->Enable(TRUE);
//	else
//		pCmdUI->Enable(FALSE);
}

BOOL TRuleView::DestroyWindow() 
{
	return CEditView::DestroyWindow();
}

BOOL TRuleView::OpenStandardFile(LPCTSTR filename) 
{
	// it is tried to open the standard rule file
	m_strFileName = filename;
	if (OpenFile(FALSE))
	{
		// succeeded, so try to parse the rules
		m_bHaveStandardFile = TRUE;
		OnParseRules();
		return TRUE;
	}

	// could not open standard rule file
	m_bHaveStandardFile = FALSE;
	m_strFileName = "";
	return FALSE;
}

BOOL TRuleView::OpenCurrentFile() 
{
	// it is tried to open the current rule file again
	if (OpenFile(FALSE))
	{
		// succeeded, so try to parse the rules
		OnParseRules();
		return TRUE;
	}

	// could not open rule file
	m_strFileName = "";
	return FALSE;
}

void TRuleView::SetBehaviorNetwork(TBehaviorNetwork* pKnow) 
{
	m_pKnowledge = pKnow;
}

void TRuleView::SetNoGraphics(BOOL val) 
{
	m_bNoGraphics = val;
}
