// TAccessMap.cpp: implementation of the TAccessMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Basis.h"
#include "TScreenParams.h"
#include "TAccessMap.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TAccessMap::TAccessMap()
{
	m_tViewDir = 0;
	m_playerSize = PLAYER_SIZE;
	m_kickableMargin = KICKABLE_MARGIN;

	// we will need a lot of similar cos values, so store them for efficiency
	// from -180 to 180
	for (int i = -NO_OF_ANGLES / 2, j = 0; j < NO_OF_ANGLES; i++, j++)
		m_dCos[j] = cos((double)(i * ANGLE_STEP) * DEG2RAD);
}

TAccessMap::~TAccessMap()
{
}


void TAccessMap::SetParameters(double kickableMargin, double playerSize)
{
	m_kickableMargin = kickableMargin;	
	m_playerSize = playerSize;
}

void TAccessMap::Initialize(TVector pos, int iAbsViewDir, double dMaxDist)
{
	int	iAbsIndex;
	int	iCosIndex90 = NO_OF_ANGLES / 4;
	
	m_tViewDir = iAbsViewDir;
	m_tPos = pos;

	iAbsIndex = GetAbsIndexFromIndex(0);
	for (int i = 0; i < NO_OF_ANGLES; i++)
	{
		// the accessible distance is initialized by the distance to the
		// outside line in the corresponding direction
		// we need to use absolute directions
		int		iRelevantIndex;
		double	dDistLine;
		
		// initialize map
		m_dAccess[i] = dMaxDist;

		// check right line
//		dDistLine = 52.5 - m_tPos.GetX();
		dDistLine = 50.0 - m_tPos.GetX();
		iRelevantIndex = iAbsIndex;
		AddFieldDistance(i, dDistLine, iRelevantIndex);

		// check bottom line
//		dDistLine = 34.0 - m_tPos.GetY();
		dDistLine = 33.0 - m_tPos.GetY();
		iRelevantIndex = iAbsIndex - iCosIndex90;
		AddFieldDistance(i, dDistLine, iRelevantIndex);

		// check left line
//		dDistLine = m_tPos.GetX() + 52.5;
		dDistLine = m_tPos.GetX() + 50.0;
		iRelevantIndex = iAbsIndex - 2 * iCosIndex90;
		if (iRelevantIndex < 0)
			iRelevantIndex += NO_OF_ANGLES;
		AddFieldDistance(i, dDistLine, iRelevantIndex);

		// check top line
//		dDistLine = m_tPos.GetY() + 34.0;
		dDistLine = m_tPos.GetY() + 33.0;
		iRelevantIndex = iAbsIndex + iCosIndex90;
		AddFieldDistance(i, dDistLine, iRelevantIndex);

		// advance angle
		iAbsIndex++;
		if (iAbsIndex >= NO_OF_ANGLES)
			iAbsIndex = 0;
	}
}

void TAccessMap::AddPlayer(double dist, int dir, int bodyDir, BOOL isGoalie)
{
	// adds a new player to the map. All access directions
	// within +-90 degerees are affected.
	double	dDistLeft = dist / 2.0 - (m_kickableMargin + m_playerSize + 1.0);
	double	dDistRight;
	double	newDist;
	int		iPlayerIndex = GetIndexFromAngle(dir);
	int		iIndexRight = iPlayerIndex;
	int		iIndexLeft = iPlayerIndex;
	int		iAbsIndex1 = NO_OF_ANGLES / 2;
	int		i = 0;

	if (isGoalie)
		dDistLeft -= 1.0;

	if (dDistLeft < 0.1)
		dDistLeft = 0.1;
	dDistRight = dDistLeft;

	while(i < NO_OF_ANGLES / 4)
	{
		NormalizeIndex(iIndexRight);
		NormalizeIndex(iIndexLeft);
		NormalizeIndex(iAbsIndex1);
		
		// at 20 degrees we take body dir of opponent into account
		// if opponent is near
		if ((i == 2) && (dist < 7.0))
		{
			if (bodyDir > 0)
				// opponent looks to my right
				dDistLeft += 2.0;
			else
				// opponent looks to my left
				dDistRight += 2.0;
		}

		// what is the accessible distance in the current direction
		newDist = dDistRight / m_dCos[iAbsIndex1];
		AddNewDistance(iIndexRight, newDist);
		newDist = dDistLeft / m_dCos[iAbsIndex1];
		AddNewDistance(iIndexLeft, newDist);

		iIndexRight++;
		iIndexLeft--;
		iAbsIndex1++;
		i++;
	}
}

void TAccessMap::AddPlayerDir(double dist, int dir)
{
	// the difference to AddPlayer is that only few directions 
	// around the player (2 * iHowMuch) are affected
	int		iHowMuch = 4;
	int		iPlayerIndex = GetIndexFromAngle(dir) - iHowMuch;
	double	deltaDist;

	for (int i = 0; i < iHowMuch * 2 + 1; i++)
	{
		NormalizeIndex(iPlayerIndex);
		
		// what is the accessible distance in the current direction
		deltaDist = (i - iHowMuch);
		deltaDist = (deltaDist * deltaDist) / 2.0;
		AddNewDistance(iPlayerIndex, dist + deltaDist);
		iPlayerIndex++;
	}
}


void TAccessMap::DrawMap(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CPen		pen;
	CPen		*oldPen;
	int		iX, iY;
	int		iOriginX;
	int		iOriginY;
	double	endX;
	double	endY;
	int		iCosIndex90 = NO_OF_ANGLES / 4;
	int		iAngle;
	double	dAngle;

	// set color
	cCol = COLOR_BALL;
	pen.CreatePen(PS_SOLID, 1,cCol);
	oldPen = pDC->SelectObject(&pen);

	GetCoordinates(tScreen, m_tPos.GetX(), m_tPos.GetY(), iOriginX, iOriginY);
	for (int i = 0; i < NO_OF_ANGLES; i++)
	{
		iAngle = GetAngleFromIndex(GetAbsIndexFromIndex(i));
		dAngle = (double) iAngle * DEG2RAD;

		// calculate the endpoints of the arrow
		endX = m_tPos.GetX() + m_dAccess[i] * cos(dAngle); 
		endY = m_tPos.GetY() + m_dAccess[i] * sin(dAngle); 
		GetCoordinates(tScreen, endX, endY, iX, iY);

		// draw all arrows
		pDC->MoveTo(iOriginX, iOriginY);
		pDC->LineTo(iX, iY);
	}
	
	pDC->SelectObject(oldPen);
	pen.DeleteObject();
}


int TAccessMap::GetIndexFromAngle(int angle)
{
	// normalize from -179 to 180
	TDirection tAngle = angle;
	// now from 0 to 359
	int	index = (int) tAngle + 179;
	index /= ANGLE_STEP;
	return index;
}

int TAccessMap::GetAbsIndexFromIndex(int index)
{
	// calculates the absolute index of the absoulte direction by
	// taking into account the view direction of the agent
	ASSERT((index >= 0) && (index < NO_OF_ANGLES));
	int iAbsIndex = index + (int)m_tViewDir / ANGLE_STEP;
	NormalizeIndex(iAbsIndex);
	return iAbsIndex;
}

int TAccessMap::GetAngleFromIndex(int index)
{
	ASSERT((index >= 0) && (index < NO_OF_ANGLES));
	int angle = index * ANGLE_STEP;
	return angle - 180;
}

double TAccessMap::GetDistanceFromIndex(int index)
{
	ASSERT((index >= 0) && (index < NO_OF_ANGLES));
	return m_dAccess[index];
}

BOOL TAccessMap::AddNewDistance(int index, double dist)
{
	// a new distance is only added, if it is closer than last one
	ASSERT((index >= 0) && (index < NO_OF_ANGLES));
	if (dist < m_dAccess[index])
	{
		m_dAccess[index] = dist;
		return TRUE;
	}
	return FALSE;
}

void TAccessMap::AddFieldDistance(int index, double dist, int iCosIndex)
{
	double	dDistAngle;
	int		iCosIndex90 = NO_OF_ANGLES / 4;

	if ((iCosIndex > iCosIndex90)
		&& (iCosIndex < iCosIndex90 * 3))
	{
		// there is an intersection
		dDistAngle = dist / m_dCos[iCosIndex];
		AddNewDistance(index, dDistAngle);
	}
}

void TAccessMap::GetCoordinates(TScreenParams* tScreen, double dx, double dy, int &x, int &y)
{
	if ((tScreen->GetTime() > 3000) && (tScreen->GetTime() <= 6000))
	{
		dx = -dx;
		dy = -dy;
	}

	// first half or extended time
	x = (int) (((double) dx + PITCH_MARGIN + PITCH_LENGTH / 2.0) * tScreen->GetAbsScale());
	y = (int) (((double) dy + PITCH_MARGIN + PITCH_WIDTH / 2.0)	* tScreen->GetAbsScale());
}

int TAccessMap::NormalizeIndex(int &index)
{
	// checks and adapts the range of an index
	if (index < 0)
		index = NO_OF_ANGLES + index;
	else if (index >= NO_OF_ANGLES)
		index -= NO_OF_ANGLES;
	return index;
}