// TCoachAgent.h: interface for the TCoachAgent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TCOACHAGENT_H__54780FF3_9868_40BF_AC6C_7F935D18A4D4__INCLUDED_)
#define AFX_TCOACHAGENT_H__54780FF3_9868_40BF_AC6C_7F935D18A4D4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TDoingAgent.h"

// for online coach
#define NO_OF_TRADEOFFS		5
#define NO_OF_PLAYERTYPES	7


// training parameters
// no of tries for one genetic type
#define TRIES_PER_TYPE		60
// no of cycles for one single test
#define TRAINING_DURATION	30



class CBasisDoc;
class CBasisView;
class TPerceptionView;
class TNetworkView;
class TRuleView;
class GeneticAlgorithm;
class GeneticCode;
class DGeneticPoolInfo;

class PlayerTypeEvaluation : public CObject
{
public:
	double	m_value;
	int		m_type;
	PlayerTypeEvaluation(int type, double value)
	{
		m_type = type;
		m_value = value;
	}
};

class TCoachAgent : public TDoingAgent  
{
	public:
		TCoachAgent(CBasisDoc* pDoc, CBasisView* pBView, TPerceptionView* pPView
						, TNetworkView* pNView, TRuleView* pRView);
		virtual ~TCoachAgent();

		virtual void OnReceivedMessage(LPCTSTR message, BOOL bFromNetwork = TRUE);
		virtual void ReceivedTerminate();
		virtual void CreatePerceptions();
		virtual void CreateResources();
		virtual void CreateActions();
		virtual BOOL DoReflexSelection();
		virtual BOOL IsCoach() { return TRUE; };
		
		double	GetHighestQuality() { return m_highestQuality; };
		CString	GetGeneticPoolInfo();
		void	ShowGeneticPoolInfo();
		void	InfoDialogUpdate();


		// perceptions
		double	GetMaySaySomething();
		double	GetWin();

		double	GetReadyForAction();
		double	GetActionFinished();

		// resources
		double	GetMouthsAvailable();

		// actions
		void	SayStrategy(double activation);

		void	EvaluateAction(double activation);
		void	StartAction(double activation);
		void	StartKick(double activation);

	private:
		void	EvaluatePlayerTypes();
		void	SwitchPlayerType(int who);
		void	InsertSorted(CObList& list, PlayerTypeEvaluation* module);
		void	EvaluateCode();
		void	UseNewCode();
		void	TellCode();
		void	WriteToTrainingLog(CString buffer);

		BOOL	m_hasEyesOn;
		BOOL	m_hasTypesEvaluated;
		int		m_noOfPlayersChanged;
		CObList	m_evaluations[PLAYERS_PER_TEAM];
		int		m_typeAvailable[NO_OF_PLAYERTYPES];

		// for training
		BOOL	m_bReadyForAction;
		int		m_actionStartTime;
		int		m_actionCount;
		double	m_highestQuality;
		double	m_averageQuality;
		double	m_varianz;

		GeneticAlgorithm*	m_genetics;
		GeneticCode*		m_currentCode;
		GeneticCode*		m_bestCode;
		double*				m_bestQualityCurve;
		double				m_bestQuality;
		int					m_populationCount;		

		CFile*				m_trainingResultFile;
		DGeneticPoolInfo*	m_infoDialog;
};

#endif // !defined(AFX_TCOACHAGENT_H__54780FF3_9868_40BF_AC6C_7F935D18A4D4__INCLUDED_)
