// TDoingAgent.h: interface for the TDoingAgent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TDOINGAGENT_H__B6B1C1C2_8CBF_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TDOINGAGENT_H__B6B1C1C2_8CBF_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TViewingAgent.h"

#ifdef _DEBUG
	// define to see some perception variables in output window
	#define xSHOW_DOING_VAR
#endif

#define HARD_DRIBBLE				15
//#define SECURE_DRIBBLE				10
#define CLOCKWISE					-1
#define COUNTERCLOCKWISE			1


// for genetics
// indices of genetic code

#define FIRST_KICK_POWER_INDEX		0
#define SECOND_KICK_POWER_INDEX		1
#define MIN_KICK_ANGLE_INDEX		2
#define SECURE_DRIBBLE_INDEX		3
#define DRIBBLE_ANGLE_INDEX			4
#define CLOSE_DRIBBLE_POWER_INDEX	5

// initial values
/*
// manual values
#define FIRST_KICK_POWER			5
#define SECOND_KICK_POWER			40
#define MIN_KICK_ANGLE				110
#define SECURE_DRIBBLE				10
#define DRIBBLE_ANGLE				0
#define CLOSE_DRIBBLE_POWER			0.2
*/
// learned values
#define FIRST_KICK_POWER			5
#define SECOND_KICK_POWER			13
#define MIN_KICK_ANGLE				116
#define SECURE_DRIBBLE				7
#define DRIBBLE_ANGLE				-48
#define CLOSE_DRIBBLE_POWER			0.12

// range within which to mutate the values (necessary because of 0 parameters)
#define FIRST_KICK_POWER_RANGE		10
#define SECOND_KICK_POWER_RANGE		80
#define MIN_KICK_ANGLE_RANGE		110
#define SECURE_DRIBBLE_RANGE		30
#define DRIBBLE_ANGLE_RANGE			80
#define CLOSE_DRIBBLE_POWER_RANGE	0.2

#define MIN_BALL_SPEED_INDEX		0
#define STOP_KICK_POWER_INDEX		1
#define MIN_TURN_DIR_INDEX			2

#define MIN_BALL_SPEED				0.4
#define	STOP_KICK_POWER				50
#define	MIN_TURN_DIR				20

#define MIN_BALL_SPEED_RANGE		1.0
#define STOP_KICK_POWER_RANGE		100
#define MIN_TURN_DIR_RANGE			60

class CBasisDoc;
class CBasisView;
class TPerceptionView;
class TRuleView;
class TNetworkView;
class TNetworkParams;
class TAct;
class TPercept;
class TVector;
class GeneticCode;

/*-----------------------------------------------------------------------------
Class   : TDoing Agent
Date    : 14.01.98
-------------------------------------------------------------------------------
Base Class: TViewingAgent
Description: adds the knowledge things to an agent
-----------------------------------------------------------------------------*/

class TDoingAgent : public TViewingAgent  
{
	public:
		TDoingAgent(CBasisDoc* pDoc, CBasisView* pBView, TPerceptionView* pPView
						, TNetworkView* pNView, TRuleView* pKView);
		virtual ~TDoingAgent();

		// some more complex actions
		void ComplexDash(double dDistance, int iDirection, int iPower = 0, BOOL bMindBall = FALSE, BOOL checkSourrounding = TRUE);
		void ComplexKick(int iPower, int iDirection, bool bQueue = false);
		void ComplexDashAndKick(int iPower, double dist, int iDirection);
		BOOL ComplexDashTo(TVector& pos, int power = 100, int dist = 3, BOOL bMindBall = FALSE, BOOL checkSourrounding = TRUE);
		BOOL ComplexDashTo(TVisibleObject* pObj, int power = 100, int dist = 3, BOOL bMindBall = FALSE, BOOL checkSourrounding = TRUE);
		void ComplexKickTo(TVector& pos, BOOL bStopThere = FALSE);
		void ComplexKickTo(TVisibleObject* pObj, BOOL bStopThere = FALSE);
		void StopBall();
		void HardKick(int iDirection);
		void OldHardKick(int iDirection);
		BOOL WillHaveBallNextStep();
		void GoalieMove(TVector pos, int dir);
		void AdjustGoaliePosition(TVector tFutureBall);
		
		virtual void		TurnNeck(int iDir, BOOL bQueue = FALSE);
		virtual void		SetKnowledgeType(int type);
		virtual void		OnReceivedMessage(LPCTSTR message, BOOL bFromNetwork = TRUE);
		virtual void		ReceivedTerminate();
		void				OnAgentReady();
		void				OnReceivedSee();
		void				OnReceivedLastSenseBody();
		void				OnTimer(BOOL bActOnSee);

		void				SetGoalTrackingOn(BOOL bVal);
		BOOL				GetGoalTrackingOn() const;
		void				SetUseFuzzyValues(BOOL val);
		BOOL				GetUseFuzzyValues() const; 
		void				SetFuzzyRound(double val);
		void				SetAdaptActivationAreaOn(BOOL bVal);
		BOOL				GetAdaptActivationAreaOn() const;

		void				LookToBall();
		CPoint				CalculateDefendingPosition();

		void				SetCombo(CComboBox* pCombo, BOOL bWhich);
		CString				GetActionPerformedToString();
		CString				GetResults();
		CString				GetResultsInitial() const;
		BOOL				IsAutorun() const;
		int					GetPosY();
		TVector				GetIntersectionPoint(int power, int turnDelay = 1);

		virtual void		SetGeneticCode(TParseString& codeString);

		// --------------------------------------------------
		// action functions (sorted alphabetically !!)

		virtual void	CreateActions();
		void	DummyAction(double activation) {};
		
		void	DoANothing(double activation);
		void	DoAwaitGoalkick(double activation);
		void	DoCatchBall(double activation);
		void	DoCheckOffside(double activation);
		void	DoKickAwayBall(double activation);
		void	DoKickGoal(double activation);
		void	DoLookStraight(double activation);
		void	DoLookToBall(double activation);
		void	DoMindBall(double activation);
		void	DoMoveToHomePosition(double activation);
		void	DoPassBall(double activation);
		void	DoProtectBall(double activation);
		void	DoPushBackward(double activation);
		void	DoPushForward(double activation);
		void	DoRelax(double activation);
		void	DoRetreat(double activation);
		void	DoRunFree(double activation);
		void	DoRunForwardToBall(double activation);
		void	DoRunToBall(double activation);
		void	DoRunToBallGoalie(double activation);
		void	DoRunToHomePosition(double activation);
		void	OldDoRunToHomePosition(double activation);
		void	DoRunToOtherGoal(double activation);
		void	DoRunToOwnGoal(double activation);
		void	DoRunWithBall(double activation);
		void	DoSayPosition(double activation = 1.0);
		void	DoSearchBall(double activation = 1.0);
		void	DoSearchFreeTeammate(double activation);
		void	DoStopBall(double activation);
		void	DoTurnAroundBallForKickIn(double activation);
		void	DoTurnToBall(double activation);
		void	DoWaitForKickOff(double activation);

//		void	BallDribbleToEnemyGoal(bool dribbleOnTheLeftSide);
//		bool	BallRotateLeftOrRight(TDirection deltaBallStopAngle);
//		void	BallRotateAroundPlayer(TDirection deltaBallStopAngle, bool rotateRight);
		// --------------------------------------------------

		double	GetAvgDistance() const { return m_dAvgDistance; };

		// for blocksworld
		BOOL			IsBlocksworld();
		BOOL			PerformCycle(TNetworkParams* params, int iNoOfCycles);
		void			StopAgent();

	protected:

		virtual BOOL		DoReflexSelection();
		BOOL				IsFreekickOther();

		CBasisDoc*			m_pDoc;
		GeneticCode*		m_geneticCode;
		BOOL				m_bAgentIsReady;
		BOOL				m_bReceivedSee;
		BOOL				m_bSelectedReflex;
		BOOL				m_bCommandDone;
		BOOL				m_bSayDone;
		BOOL				m_bWantNarrowView;
		BOOL				m_bWantWideView;
		BOOL				m_bHaveActedThisCycle;
		int					m_iSenseBodyCount;

		// for statistics
		double	m_dAvgDistance;
		long	m_lNoOfRelaxes;
		int		m_iNoOfCycles;
		int		m_iNoOfConcurrentCycles;
		int		m_iNoOfLazyCycles;

	private:
		void		AccountBallSpeed(int& power, TDirection& dir, int& resultPower, int& resultDir);
		BOOL		DribbleCounterclockwise(int power);
		BOOL		DribbleClockwise(int power);
		BOOL		MoveBallToPoint(int bDirection, int iOrientation);
		int			GetDirectionPreciseness(double dBallDistance);
		BOOL		GetIsBehind(double dist, int dir);
		BOOL		GetIsInFront(double dist, int dir);
		void		SurroundOpponent(int dir);
		BOOL		CheckSourrounding();
		BOOL		OldCheckSourrounding();
		TVector		CalculateKick(double dir,double bx, double by,double mf);
		TVector		GoalieRunToBall(TVector tFutureBall);
		TVector		DefenderRunToBall(TVector tFutureBall, int playerX, int playerY);
		int			PushForwardPower(int stamina);

};

#endif // !defined(AFX_TDOINGAGENT_H__B6B1C1C2_8CBF_11D1_B242_0000C09B5CBE__INCLUDED_)
